---
title: Como configurar e usar a API do S3 junto com o Ceph
tags: [ceph, s3]
---

A instância do Ceph do C3SL possui as seguintes valores:

| Informação | Valor                     |
| ---------- | ------------------------- |
| URL        | <https://s3.c3sl.ufpr.br> |
| Região     | default                   |

## Instalação

O primeiro passo é instalar a cli da AWS:

```console
$ apt install awscli
```

O próximo passo é configurar a cli:

```console
$ aws configure
AWS Access Key ID: <key_id>
AWS Secret Access Key: <secret_key:
Default region name [None]: <zone>
Default output format [None]: <tipo>
```

Note que atualmente o nome da região é `default`.
Você pode encontrar o nome da região pelo comando:

```console
$ radosgw-admin zonegroup list
```

Por exemplo:

```console
barril1:~# radosgw-admin zonegroup list
{
    "default_info": "3a9fb087-0439-4632-8eb8-818c5447fd71",
    "zonegroups": [
        "default"
    ]
}
```

Exemplo do aws configure:

```console
root@docs:$ aws configure
AWS Access Key ID [********************]:
AWS Secret Access Key [********************]:
Default region name [None]: default
Default output format [None]: json
```

## Comandos de listagens

Para listar todos os buckets de um usuário já configurado use o
comando: `aws --endpoint-url=<url> s3api list-buckets`. Por exemplo:

```console
root@docs:~# aws --endpoint-url=https://s3.c3sl.ufpr.br s3api list-buckets
{
    "Buckets": [
        {
            "Name": "my-bucket",
            "CreationDate": "2022-08-12T00:36:36.028Z"
        }
    ],
    "Owner": {
        "DisplayName": "Docs-rdt",
        "ID": "docsRDT"
    }
}
```

Para adicionar um bucket use o comando
`aws --endpoint-url=<url> s3api create-bucket --bucket <nome_bucket> --region <region>`.
Existem algumas opções adicionais, como passar ou não a ACL no momento da criação.

Por exemplo, para criar um bucket simples, sem ACL:

```console
root@docs:~# aws --endpoint-url=https://s3.c3sl.ufpr.br s3api create-bucket --bucket media --region default
root@docs:~# aws --endpoint-url=https://s3.c3sl.ufpr.br s3api list-buckets
{
    "Buckets": [
        {
            "Name": "my-bucket",
            "CreationDate": "2022-08-12T00:36:36.028Z"
        },
        {
            "Name": "media",
            "CreationDate": "2022-08-12T00:53:15.871Z"
        },
    ],
    "Owner": {
        "DisplayName": "Docs-rdt",
        "ID": "docsRDT"
    }
}
```

É possível definir a ACL do bucket no momento da criação com adição da
opção `--acl` com as opções:

- private
- public-read
- public-read-write
- authenticated-read

Por exemplo:

```console
root@docs:~# aws --endpoint-url=https://s3.c3sl.ufpr.br s3api create-bucket \
                                --bucket static \
                                --region default \
                                --acl public-read
root@docs:~# aws --endpoint-url=https://s3.c3sl.ufpr.br s3api list-buckets
{
    "Buckets": [
        {
            "Name": "media",
            "CreationDate": "2022-08-12T00:53:15.871Z"
        },
        {
            "Name": "my-bucket",
            "CreationDate": "2022-08-12T00:36:36.028Z"
        },
        {
            "Name": "static",
            "CreationDate": "2022-08-12T01:03:05.926Z"
        }
    ],
    "Owner": {
        "DisplayName": "Docs-rdt",
        "ID": "docsRDT"
    }
}
```

Para remover um bucket utilize o comando:
`aws --endpoint-url=<url> s3api delete-bucket --bucket <bucket>`.
Por exemplo:

```console
root@docs:~# aws --endpoint-url=https://s3.c3sl.ufpr.br s3api delete-bucket --bucket my-bucket
root@docs:~# aws --endpoint-url=https://s3.c3sl.ufpr.br s3api list-buckets
{
    "Buckets": [
        {
            "Name": "media",
            "CreationDate": "2022-08-12T00:53:15.871Z"
        },
        {
            "Name": "static",
            "CreationDate": "2022-08-12T01:03:05.926Z"
        }
    ],
    "Owner": {
        "DisplayName": "Docs-rdt",
        "ID": "docsRDT"
    }
}
```

Para remover todos os objetos de um bucket:

```bash
export bucket=media
export acl=public-read
export endpoint=https://s3.c3sl.ufpr.br
export objects=$(aws --endpoint-url=${endpoint} s3api list-objects --bucket ${bucket} |\
        grep -i key | cut -d':' -f2 | sed 's/,//' | sed 's/"//g')

for object in ${objects}; do
    aws --endpoint-url=${endpoint} s3api delete-object --bucket ${bucket} --key ${object}
done;
```

Para alterar a ACL de um bucket use o comando
`aws --endpoint-url=<url> s3api put-bucket-acl --bucket <bucket> --acl <acl>`.
**Onde:**

- **\<url>**: URL do endpoint do S3.
- **\<bucket>**: nome do bucket.
- **\<acl>**: tipo de ACL, podendo ser os seguintes valores:
  * public-read
  * private
  * public-read-write
  * authenticated-read

Para alterar a ACL de todos os objetos de um bucket, no caso para public-read:

```bash
bucket=media
acl=public-read
endpoint=https://s3.c3sl.ufpr.br
objects=$(aws --endpoint-url=${endpoint} s3api list-objects --bucket ${bucket} |\
        grep -i key | cut -d':' -f2 | sed 's/,//' | sed 's/"//g')

for object in ${objects}; do
    aws --endpoint-url=${endpoint} s3api put-object-acl --bucket ${bucket} --key ${object}  --acl ${acl};
done;
```

## Politica

Considere o seguinte json:

```console
{
    "Statement": [
        {
            "Sid": "PublicRead",
            "Effect": "Allow",
            "Principal": "*",
            "Action": [
                "s3:GetObject",
                "s3:GetObjectVersion"
            ],
            "Resource": [
                "arn:aws:s3:::media/*"
            ]
        }
    ]
}
```

Salve esse json com nome de `policy.json` e para adicionar no bucket use o comando:

```console
aws --endpoint https://s3.c3sl.ufpr.br s3api put-bucket-policy --bucket media --policy file://policy.json
```

Cuidado ao definir políticas: não deixe as coisas mais acessíveis do que deveriam.
Nesse exemplo, o bucket tem como única função servir arquivos estáticos.
