---
title: Como configurar Fail2Ban para WordPress
---

Configuração do fail2ban nas máquinas c3slprod e webinfpro com o propósito de
prevenir vulnerabilidades causadas pelo WordPress.

## Instalação

Através do `apt`:

```console
$ apt install -y fail2ban
```

Uma vez instalado, pode-se observar o estado do serviço:

```console
$ systemctl status fail2ban
```

Se tudo ocorreu conforme o esperado, devemos ver um output mais ou menos desta forma:

```console
$ systemctl status fail2ban
● fail2ban.service - Fail2Ban Service
    Loaded: loaded (/lib/systemd/system/fail2ban.service; enabled; vendor preset: enabled)
    Active: active (running) since Mon 2021-05-24 12:08:07 UTC; 3min 5s ago
      Docs: man:fail2ban(1)
  Main PID: 341 (f2b/server)
     Tasks: 7 (limit: 464145)
    Memory: 14.6M
    CGroup: /system.slice/fail2ban.service
            └─341 /usr/bin/python3 /usr/bin/fail2ban-server -xf start

May 24 12:08:07 ubuntu systemd[1]: Starting Fail2Ban Service...
May 24 12:08:07 ubuntu systemd[1]: Started Fail2Ban Service.
May 24 12:08:08 ubuntu fail2ban-server[341]: Server ready
```

## Configuração de regra para o WordPress

Arquivos `*.conf` são configurações padrão e não devem ser alterados.
Todas as alterações devem ser feitas em arquivos `*.local`.

Portanto, a configuração das jails foi feita em `/etc/fail2ban/jail.local`,
da seguinte forma:

```ini
# As configuracoes default estao em jail.conf, que não deve ser alterado.
#
# As configurações presentes neste arquivo sobrepoem (override) aquelas
# do arquivo jail.conf.
#
# Iago, Nov 2022

[DEFAULT]

# tres tentativas com erro em 60 minutos -> banimento do IP
maxretry = 3
findtime = 30m

# tempo de banimento
bantime = 5m
bantime.increment = true
bantime.maxtime = 30d

# ação default é ICMP reject, mas DROP gasta mais recursos do atacante
banaction = iptables-multiport[blocktype=DROP]

# IPs ignorados (copiado de postfix/main.cf)
ignoreip = 127.0.0.0/8 ::1/128 10.254.229.0/24 10.254.222.17/32 200.17.202.0/25 200.17.202.188/32 200.236.31.128/25 200.236.31.139/32 2801:0082:80ff:8000::1/64 2801:0082:80ff:8001::1/64 2801:0082:80ff:8002::1/64

[sshd]
enabled = false

# wordpress
[wordpress]
enabled = true
port = http,https
filter = wordpress
action = iptables-multiport[name=wordpress, port="http,https", protocol=tcp]
logpath = /var/log/nginx/*access.log
maxretry = 3
bantime = 3600
```

Os filtros estão no diretório `/etc/fail2ban/filter.d/`.
Para criar um filtro, crie um arquivo neste diretório.

Para o WordPress, foi criado um arquivo `/etc/fail2ban/filter.d/wordpress.conf`,
contendo:

```ini
[Definition]
failregex = ^<HOST> .* "(GET|POST) /wp-login.php
            ^<HOST> .* "(GET|POST) /xmlrpc.php
ignoreregex =
```

Este filtro vai olhar requisições GET e POST.

Após isto, basta fazer o restart do serviço:

```console
$ systemctl restart fail2ban
```
