---
title: Como fazer a comutação entre bancos de dados
---

Atualmente, o C3SL conta com duas máquinas de banco de dados:

:::{list-table}
* - **Nome**
  - C3SLDB1
  - C3SLDB2
* - **Modelo**
  - SGI antiga
  - Intel
* - **Núcleos**
  - 16
  - 20
* - **RAM**
  - 128G
  - 256G
* - **Sistema de arquivos**
  - ZFS
  - ZFS
* - **Discos**
  - 6x6 TB RAIDZ2
  - 6x6 TB RAIDZ2
* - **Cache**
  - 1 SSD SATA para l2arch
  - 300G NVME Samsung para l2arch
* - **Cache de escrita**
  - Indexes e tabelas do Moodle armazenada diretamente no NVME
  - Indexes e tabelas do Moodle armazenada nos discos
:::

Com base na configuração das duas servidoras de banco de dados, é notável a
superioridade da servidora C3SLDB2. Portanto, recomenda-se ela ser a servidora
principal, enquanto a servidora c3sldb1 fique como secundária em caso de falha
na c3sldb2.

## Processo de comutação entre servidoras

Caso a servidora principal falhe é necessário seguir determinados procedimentos para que a
servidora secundária possa assumir. O tempo estimado para executar todo o procedimento é de aproximadamente 30 minutos. Os passos a seguir são:

### Passo 1: Garantir que o arquivo `pg_hba` esteja atualizado

Como tem duas servidoras de bancos de dados, acontecerá eventualmente de adicionar
um novo host ao `pg_hba` da servidora principal e não adicionar na servidora
secundária, logo, se a servidora principal falhar, as aplicações que não
estão no `pg_hba` da servidora secundária não vão funcionar.

É extremamente recomendável utilizar um gerenciador de configuração que atualiza o `pg_hba` tanto na servidora principal quanto na secundária, para deixar síncrono. Algumas opções são:

- **Ansible** por sua sintaxe simples, baixa curva de aprendizado e de baixa complexidade.
- **Salt com minion** O Salt com minion possui uma funcionalidade que monitora o
  arquivo das servidoras. Então, por exemplo, eu aplicar de manhã o estado Salt para o `pg_hba` na servidora principal, o minion do Salt vai identificar a mudança e que não condiz com a versão que está no servidor do Salt e irá sobrescrever imediatamente. Isso previne que usuários avacalhem com os arquivos de configuração. Visto que o Salt sempre vai garantir que os arquivos de configuração são os mesmos que estão no servidor do Salt.

### Passo 2: Configurar a servidora principal para ser tornar servidora secundária

Esse passo só se aplica em momento necessário realizar configuração na servidora principal ou quando a servidora principal volta a funcionar. Nos testes realizados hoje, 2024-02-13, foi necessário alterar o arquivo `/etc/postgres/15/main/postgres.conf` e colocar a configuração de secundário.

É de extrema importância automatizar esse processo, por ter um certo nível de complexidade e banco de dados críticos. E a maior falha de sistema distribuído normalmente é cometido por humanos.

### Passo 2.1: Configuração do servidor principal

O arquivo de configuração do servidor principal são as seguintes opções:

```ini
wal_level = replica # essa opção não muda entre principal e segundário
max_wal_senders = 10 # essa opção não muda entre principal e segundário
archive_mode = on # essa opção não muda entre principal e segundário

# Essas duas linhas abaixo define que o servidor será principal e que ele fará scp do wal log para servidora secundária, no caso c3sldb1

archive_command = 'test ! -f /home/postgres/archive/%f && scp -o "StrictHostKeyChecking no" %p c3sldb1.c3sl.ufpr.br:/home/postgres/archive/%f'
archive_cleanup_command = '/usr/bin/pg_archivecleanup -d /home/postgres/15/main/pg_wal/ %r >> /var/log/postgresql/archive_cleanup.log 2>&1'
```

### Passo 2.2: Configuração do DNS

Após configurar o Postgres, mas sem antes reiniciar o serviço é necessário entrar
na urquell e mudar o DNS para nova servidora. O arquivo que deve ser alterado
é o `/etc/bind/c3sl.zone`, especificamente as linhas:

```
postgres 30 IN A 200.17.202.30
postgres 30 IN AAAA 2801:82:80ff:8001:260:ddff:fe46:53cd
```

note que o endereço IP depende de qual servidora será a master.
A tabela a seguir indica os endereços da c3sldb1 e c3sldb2:

| Nome    | IPv4          | IPv6                                  |
| ------- | ------------- | ------------------------------------- |
| c3sldb1 | 200.17.202.97 | 2801:82:80ff:8001:92e2:baff:fe2b:3aa0 |
| c3sldb2 | 200.17.202.30 | 2801:82:80ff:8001:260:ddff:fe46:53cd  |

Após configurar o endereço IP do nome `postgres.c3sl.ufpr.br` é
necessário executar o script `/root/bin/atualiza-dns-dhcp.py` para atualizar o serial.

Note que o TTL do domínio `postgres.c3sl.ufpr.br` é de 30 segundos,
logo pode demorar 30 segundos para os clientes perceberem que foi alterado o endereço IP.

Feito as alterações na urquell é possível voltar a acertar as configurações
das servidoras de banco de dados.

### Passo 3: Configuração do servidor secundário

O arquivo de configuração do servidor secundário recebe basicamente o wal log e aplica ao banco:

```ini
# conexão com o banco principal, a senha está omitida e está localizada em ambas servidoras em
#`/root/pass`

primary_conninfo = 'host=postgres.c3sl.ufpr.br port=5432 user=replica password=pass'      # con    nection string to sending server

wal_level = replica # Essa configuração é comum tanto para primario quanto secundário
max_wal_senders = 10 # Essa configuração é comum tanto para primario quanto para secundário


archive_mode = on # essa configuração é comum tanto para primario quanto para secundário
c3sldb2.c3sl.ufpr.br:/home/postgres/archive/%f' # indica de onde vai receber o wal log para poder aplicar a replica
# realiza a limpeza de wal log que já foram processado
archive_cleanup_command = '/usr/bin/pg_archivecleanup -d /home/postgres/15/main/pg_wal/ %r >> /var/log/postgresql/archive_cleanup.log 2>&1'

restore_command = 'cp /home/postgres/archive/%f %p' # Ativar quando virar replica executa esse comando para copiar os wal level para dentro do postgres
```

### Passo 3.1: Reiniciar o servidor secundário caso ele esteja ativo

Se, por ventura, o antigo servidor primário esteja inacessível, apenas ignore essa parte e execute quando o servidor falhar. Mas basta realizar um restart no banco de dados:

```console
$ systemctl restart postgresql@15-main
```

### Passo 4: Ativar o master

Após ter configurado os arquivos do Postgres e do DNS, para tornar o servidor secundário em principal, siga os comandos abaixo:

```console
$ systemctl restart postgresql@15-main # para atualizar as configurações
$ /usr/lib/postgresql/15/bin/pg_ctl promote -D /etc/postgresql/15/main
```

Note que o comando `pg_ctl promote` recebe como entrada o diretório de configuração do Postgres, então, em eventuais upgrades, é necessário se atentar para mudar o caminho de configuração.

Encontrar o comando `pg_ctl` não foi uma tarefa trivial, o PostgreSQL tem uma política de colocar binário fora do padrão Linux (`/usr/bin`). Portanto, para não me estressar, eu sempre executo o comando:

`find / -xdev -type f -name pg_ctl` me poupa alguns anos de vida, não me estressando em procurar esse binário.

## Processo de reativar o servidor secundário

Após a falha do servidor, para recolocar no cluster do Postgres não é trivial. Na verdade, é até arcaico. Você tem que entrar na servidora secundária, no diretório onde ficam os dados do Postgres, normalmente em `/home/postgres`, remover o diretório `15/main` e daí realizar a sincronização do servidor principal com o comando:

```console
$ pg_basebackup -D 15/main/ -h postgres.c3sl.ufpr.br -X stream -c fast -U replica -W -P --wal-method=fetch
```

Isso vai copiar todo banco do servidor principal para o secundário para daí
começar a realizar a replicação.

Hoje, 2024-02-14, 00:11, isso não é problema, pois nossos bancos são pequenos e
temos boas máquinas. Mas no momento que o banco de dados tiver 10 TB de dados se
tornará impraticável.

## Notas do Odair

Replicação de serviço é um assunto muito complexo, principalmente se formos olhar o perfil de bolsistas do C3SL que normalmente estão no período inicial, sem ter realizado disciplina de Redes I, Redes II e Sistemas Distribuídos. Algumas aplicações automatizam e deixam em alto nível toda parte de sistema distribuído, como o caso do Ceph, que você não precisa saber o que é Paxos para gerenciar minimamente o Ceph.

No entanto, eu desconheço algo semelhante para Postgres. O que eu conheço como Pgpool, PgBouncer implementa em baixo nível e é necessário conhecimento mais profundo de redes e sistemas distribuídos, tornando impraticável para o C3SL.

Eu estou mais inclinado a adotar um sistema de banco de dados de cloud como o [trove](https://wiki.openstack.org/wiki/Trove). Basicamente, a gente fornece um sistema de armazenamento super rápido e cada equipe de projeto cria quantas instâncias e replicação quiserem do banco de dados. É o modelo adotado pelo RDS e pelo Aurora da AWS. Outra vantagem é que nossos bolsistas vão sair do C3SL dominando tecnologias de ponta.

Claro que nem tudo são flores, trove demanda maior dedicação da equipe de projeto, talvez cada equipe de projeto teria que ter um DevOps.

Claro, se estivéssemos no mundo da Alice, no país das maravilhas, teríamos recursos e equipe suficientes para ter time de DevOps, testar tecnologia e contribuir com a comunidade. Até mesmo publicar mais artigos, talvez no Latinoware da vida, com comparativo e contribuições para projetos de código aberto. Às vezes sinto pouco de falta de incentivo nessa parte.
