---
title: Como fazer debootstrap das Torneiras
tags: [debootstrap]
---

## Configurando RAID

```console
$ apt install debootstrap mdadm arch-install-scripts
```

Repetir nos dois discos:

```console
$ fdisk /dev/sd{X,Y}
g
n
enter
enter
+1M
t
4
n
enter
enter
enter
w
```

Depois:

```console
$ mdadm --examine /dev/sdX /dev/sdY
$ mdadm --create /dev/md0 --level=1 --metadata=1.2 --raid-devices=2 /dev/sdX2 /dev/sdY2
$ mkfs.ext4 /dev/md0
$ mount /dev/md0 /mnt
```

## Debootstrap

```console
$ debootstrap --arch=amd64 --variant=minbase bullseye /mnt http://debian.c3sl.ufpr.br/debian/

$ mount -t proc /proc /mnt/proc/
$ mount -t sysfs /sys /mnt/sys/
$ mount -o bind /dev /mnt/dev/
$ mount --bind /dev/pts /mnt/dev/pts

$ genfstab -U /mnt >> /mnt/etc/fstab

$ mdadm --detail --scan >> /mnt/etc/mdadm.conf

$ chroot /mnt /bin/bash

$ cat > /etc/apt/apt.conf.d/01norecommend << EOF
APT::Install-Recommends "0";
APT::Install-Suggests "0";
EOF

$ apt update && apt install vim htop wget
```

## Configuração do repositório

Edite o arquivo `/etc/apt/sources.list`:

```
deb http://debian.c3sl.ufpr.br/debian bullseye main
deb http://debian.c3sl.ufpr.br/debian bullseye-backports main
deb http://debian.c3sl.ufpr.br/debian bullseye-updates main
deb http://debian.c3sl.ufpr.br/debian-security/ bullseye-security main
```

## Proxmox

```console
$ echo "deb [arch=amd64] http://download.proxmox.com/debian/pve bullseye pve-no-subscription" > /etc/apt/sources.list.d/pve-install-repo.list
$ wget http://download.proxmox.com/debian/proxmox-release-bullseye.gpg -O /etc/apt/trusted.gpg.d/proxmox-release-bullseye.gpg
$ apt update && apt full-upgrade
```

## Instalação de pacotes essenciais

```console
$ apt install --no-install-recommends python3 busybox systemd systemd-sysv ifupdown2 isc-dhcp-client udev dbus vim less openssh-server locales rsync dialog openssl man iputils-* salt-minion console-setup keyboard-configuration
$ apt install --no-install-recommends proxmox-ve postfix open-iscsi
$ apt install --no-install-recommends mdadm
$ apt install --no-install-recommends grub-pc
```

## Configuração GRUB

```console
$ grub-mkconfig -o /boot/grub/grub.cfg

$ grub-install --target=i386-pc /dev/sdX
$ grub-install --target=i386-pc /dev/sdY
```

## Configuração do systemd-network

Edite `/etc/systemd/network/10-rename_interface.link`:

```ini
[Match]
# Vlan interfaces share the same base mac-address. To match only the
# real device, vlan interfaces must be excluded with Type=!vlan
MACAddress = MACADDRESS
Type = !vlan

[Link]
Name = eth0
```

## Configurações gerais

```console
$ passwd
$ dpkg-reconfigure tzdata
```

Adicionar localizações `en_US.UTF-8` e `pt_BR.UTF-8`:

```console
$ dpkg-reconfigure locales
$ mkdir /var/log/journal
$ mkdir /root/.ssh/
$ vim /etc/environment
```

Configuração do ambiente:

```console
$ HISTTIMEFORMAT="%d/%m/%y %T " # history with date
$ HISTSIZE=4000
```

## Finalização

```console
$ apt-get clean && history -c && exit

$ umount /mnt/proc
$ umount /mnt/sys
$ umount /mnt/dev/pts
$ umount /mnt/dev

$ cp /root/.ssh/authorized_keys /mnt/root/.ssh/

$ umount /mnt
```
