---
title: Como fazer debootstrap das Máquinas Virtuais
tags: [debootstrap]
---

## Install debootstrap and qemu

```console
$ sudo apt install debootstrap qemu-utils
```

## Install system

Usage: `debootstrap --arch=ARCH --variant==minbase|buildd|fakechroot|scratchbox <RELEASE> <DIR> <MIRROR>`

For Debian:

```
$ debootstrap --arch=amd64 --variant=minbase <RELEASE> /mnt http://debian.c3sl.ufpr.br/debian/
```

For Ubuntu:

```
$ debootstrap --arch=amd64 --variant=minbase <RELEASE> /mnt http://ubuntu.c3sl.ufpr.br/ubuntu
```

## Chroot into installed system

```console
$ mount -t proc /proc /mnt/proc/
$ mount -t sysfs /sys /mnt/sys/
$ mount -o bind /dev /mnt/dev/
$ mount --bind /dev/pts /mnt/dev/pts

$ chroot /mnt /bin/bash
```

## Install minimum tools to able a peaceful and IMproved work

```console
$ apt update && apt install vim htop
```

## Edit fstab file

PS: it can change accordingly to what are you doing.

```console
$ vim /etc/fstab
```

Usually, fstab must be like this for VMs:

```
#<file system>        <dir>         <type>    <options>                <dump> <pass>
/dev/vda              /             ext4      errors=remount-ro         0     1
```

## Configure apt sources

```console
$ vim /etc/apt/sources.list
```

Example for Debian:

```
deb http://debian.c3sl.ufpr.br/debian $RELEASE main
deb http://debian.c3sl.ufpr.br/debian $RELEASE-backports main
deb http://debian.c3sl.ufpr.br/debian $RELEASE-updates main
deb http://debian.c3sl.ufpr.br/debian-security/ $RELEASE/updates main
```

Example for Ubuntu:

```
deb http://ubuntu.c3sl.ufpr.br/ubuntu/ $RELEASE main restricted universe multiverse
deb http://ubuntu.c3sl.ufpr.br/ubuntu/ $RELEASE-updates main restricted universe multiverse
deb http://ubuntu.c3sl.ufpr.br/ubuntu/ $RELEASE-backports main restricted universe multiverse
deb http://ubuntu.c3sl.ufpr.br/ubuntu $RELEASE-security main restricted universe multiverse
```

## Update apt

```console
$ apt update
```

## Configure system

For virtual machines, there is no need to install linux kernel, since we pass it via qemu:

```
$ apt install rsyslog python3 busybox systemd systemd-sysv isc-dhcp-client udev dbus vim less openssh-server locales rsync dialog openssl man iputils-*
```

If you need to install the kernel:

```
$ apt install linux-image-amd64
```

## Choose timezone and locales

```console
$ dpkg-reconfigure tzdata
$ dpkg-reconfigure locales
```

## Configure systemd-networkd

Read the [Arch Linux wiki for systemd-networkd](https://wiki.archlinux.org/title/systemd-networkd)
for more information.

Usually, the following is enough for VMs.

Create the file `/etc/systemd/network/10-ens18.network` with:

```
[Match]
Name = ens18

[Network]
DHCP = yes
```

Enable the service with:

```
$ systemctl enable systemd-networkd
```

## Configure systemd-resolved

Read the [Arch Linux wiki for systemd-resolved](https://wiki.archlinux.org/title/systemd-resolved)
for more information.

Edit the file `/etc/systemd/resolved.conf` to set your prefered
DNS servers and domains.

Enable the service with:

```
$ systemctl enable systemd-resolved
```

Create the stub symbolic link:

```
$ ln -sf ../run/systemd/resolve/stub-resolv.conf /etc/resolv.conf
```

## Enable persistent journald

```console
$ mkdir /var/log/journal
```

## Set root's password

```console
$ passwd
```

## Set environment

Edit `/etc/environment`:

```
HISTTIMEFORMAT="%d/%m/%y %T " # history with date
HISTSIZE=4000
EDITOR=vim
```

## Set authorized keys

```console
$ mkdir /root/.ssh/
$ vim /root/.ssh/authorized_keys
```

## Finish installation

Optionally, you can clean apt downloaded archives:

```console
$ apt clean
```

Now exit the chroot environment by typing:

```console
$ history -c && exit
```

Umount recursively:

```console
$ umount -R /mnt
```
