---
title: Como instalar a C3SLDB
---

## Instalação do Postgres

1. O primeiro passo consiste em instalar a lista de pacotes dependentes:

   ```console
   $ export packages="curl wget sudo gnupg1 gnupg2"
   $ for package in ${packages}; do
   salt 'mvdb*' pkg.install ${package}
   done;
   ```

2. Adicione o repositório do Postgres:

   ```console
   $ salt 'mvdb*' cmd.run 'echo "deb http://apt.postgresql.org/pub/repos/apt bullseye-pgdg main" > /etc/apt/sources.list.d/pgdg.list'
   ```

3. Adicione a chave do repositório do Postgres:

   ```console
   $ salt 'mvdb*' cmd.run "wget --quiet -O - https://www.postgresql.org/media/keys/ACCC4CF8.asc | sudo apt-key add - ; apt update"
   ```

4. Instale o Postgres:

   ```console
   $ salt 'mvdb*' state.apply postgres
   ```

## Instalação do Pgpool

O Pgpool precisa ser compilado porque a versão atual no Debian está desatualizada.

1. O primeiro passo consiste em instalar as dependências:

   ```console
   $ export packages="make gcc g++ bison git flex libpq-dev libpqxx-dev clang-11 lib32gcc-s1 lib32stdc++6 libc6-i386 libclang-common-11-dev libclang-cpp11 libclang1-11 libffi-dev libgc1 libncurses-dev libobjc-10-dev libobjc4 libpfm4 libpq-dev libssl-dev libtinfo-dev libz3-dev llvm-11 llvm-11-dev llvm-11-runtime llvm-11-tools postgresql-server-dev-13"
   $ for package in ${packages}; do
   salt 'mvdb*' pkg.install ${package}
   done;
   ```

2. Obtenha o código-fonte do Pgpool:

   ```console
   $ salt 'mvdb*' cmd.run "wget --quiet -O /root/pgpool4.3.tar.gz https://www.pgpool.net/mediawiki/download.php?f=pgpool-II-4.3.0.tar.gz"
   ```

   Descompacte o código-fonte:

   ```console
   $ salt 'mvdb*' archive.tar xzvf /root/pgpool4.3.tar.gz
   ```

   Foi criado o diretório `pgpool-II-4.3.0`.

3. Entre em cada máquina no diretório `/root/pgpool-II-4.3.0` e execute:

   ```console
   $ ./configure
   $ make -j16
   $ make install
   ```

4. Verifique a versão instalada:

   ```console
   $ salt 'mvdb*' cmd.run 'pgpool --version'
   ```

## Instalação do pgpool_recovery

O pgpool_recovery é uma extensão do Postgres para realizar o recovery. É necessário ter realizado a etapa anterior antes de instalar o pgpool_recovery.

1. Compile a extensão.
   Entre em cada máquina no diretório `/root/pgpool-II-4.3.0/src/sql/pgpool-recovery`
   e execute os seguintes comandos:

   ```console
   $ make -j16
   $ make install
   ```

   Note que precisa da versão de desenvolvimento do Postgres. No caso estamos
   utilizando a versão `postgresql-server-dev-13`.

2. Instale a extensão no `template1`.
   Esse passo é mais complicado, precisa copiar o SQL `pgpool-recovery.sql`
   para dentro da home do Postgres, alterar o dono do arquivo e editar o
   arquivo, pois contém erro de sintaxe.

   1. Copiando e alterando a permissão. Em cada máquina execute:

      ```console
      $ cp /root/pgpool-II-4.3.0/src/sql/pgpool-recovery/pgpool-recovery.sql /var/lib/postgresql/
      $ chown postgres:postgres /var/lib/postgresql/pgpool-recovery.sql
      ```

   2. Agora que copiou o SQL para dentro da home do Postgres,
      mude para o usuário do Postgres, com:

      ```console
      $ su - postgres
      ```

      E edite o arquivo `pgpool-recovery.sql`. As primeiras linhas do
      arquivo estão dessa forma:

      ```sql
      CREATE FUNCTION pgpool_recovery(IN script_name text,
                 IN remote_host text,
                 IN remote_data_directory text,
                 IN primary_port text,
                 IN remote_node integer,
                 IN remote_port text)
                 IN primary_host text,
      RETURNS bool
      AS '$libdir/pgpool-recovery', 'pgpool_recovery'
      LANGUAGE C STRICT;
      ```

      Note que tem um `)` no lugar errado na linha `IN remote_port text)`.
      Altere para ficar dessa forma:

      ```sql
      CREATE FUNCTION pgpool_recovery(IN script_name text,
                 IN remote_host text,
                 IN remote_data_directory text,
                 IN primary_port text,
                 IN remote_node integer,
                 IN remote_port text,
                 IN primary_host text)
      RETURNS bool
      AS '$libdir/pgpool-recovery', 'pgpool_recovery'
      LANGUAGE C STRICT;
      ```

      Fazendo um diff do que foi alterado, deve ficar dessa forma:

      ```console
      postgres@mvdb2:~$ diff -y pgpool-recovery.old.sql pgpool-recovery.new.sql | head
      CREATE FUNCTION pgpool_recovery(IN script_name text,            CREATE FUNCTION pgpool_recovery(IN script_name text,
                 IN remote_host text,                                            IN remote_host text,
                 IN remote_data_directory text,                                  IN remote_data_directory text,
                 IN primary_port text,                                           IN primary_port text,
                 IN remote_node integer,                                         IN remote_node integer,
                 IN remote_port text,                               |            IN remote_port text)
                 IN primary_host text)                              |            IN primary_host text,
      RETURNS bool                                                    RETURNS bool
      AS '$libdir/pgpool-recovery', 'pgpool_recovery'                 AS '$libdir/pgpool-recovery', 'pgpool_recovery'
      LANGUAGE C STRICT;                                              LANGUAGE C STRICT;
      ```

      O lado esquerdo é o correto e o lado direito como estava.

   3. Agora, após editar o arquivo, basta instalar no banco `template1`:

      ```console
      $ psql -f pgpool-recovery.sql template1
      ```

3. Modifique o arquivo de configuração do Postgres. Após instalar a extensão
   no Postgres, é necessário adicionar a seguinte linha no arquivo de
   configuração do Postgres:

   ```ini
   pgpool.pg_ctl = '/usr/lib/postgresql/13/bin/pg_ctl
   ```

   Para isso, vamos editar o pillar do Postgres, no [Salt](/pages/services/salt) e
   adicionar lá a linha.

### Passos para criar `insert_lock` table

Esse passo só é necessário caso queira usar o `native-replication` do Pgpool:

1. Copie o código SQL para dentro da home do Postgres. Em cada máquina execute:

   ```console
   $ cp /root/pgpool-II-4.3.0/src/sql/insert_lock.sql /var/lib/postgresql/
   $ chown postgres:postgres /var/lib/postgresql/insert_lock.sql
   ```

2. Mude para o usuário `postgres` e execute o código `insert_lock.sql`
   sobre o banco `template1`:

   ```console
   $ su - postgres
   $ psql -f insert_lock.sql template1
   ```

## Configuração do Pgpool

O primeiro passo consiste em criar um serviço do [](/pages/concepts/systemd) para o Pgpool.
O modelo utilizado é o do código-fonte do Pgpool, com algumas alterações.
Na versão 4.3 o serviço disponível é:

```ini
[Unit]
Description=Pgpool-II
After=syslog.target network.target

[Service]
User=postgres
Group=postgres

EnvironmentFile=-/etc/sysconfig/pgpool

ExecStart=/usr/bin/pgpool -f /etc/pgpool-II/pgpool.conf $OPTS
ExecStop=/usr/bin/pgpool -f /etc/pgpool-II/pgpool.conf $STOP_OPTS stop
ExecReload=/usr/bin/pgpool -f /etc/pgpool-II/pgpool.conf reload

[Install]
WantedBy=multi-user.target
```

As alterações consistem em alterar o nome `pgpool-II` para `pgpool`.
Então o serviço deve ficar dessa maneira:

```
[Unit]
Description=Pgpool
After=syslog.target network.target

[Service]

User=postgres
Group=postgres

EnvironmentFile=-/etc/sysconfig/pgpool

ExecStart=/usr/bin/pgpool -f /etc/pgpool/pgpool.conf $OPTS
ExecStop=/usr/bin/pgpool -f /etc/pgpool/pgpool.conf $STOP_OPTS stop
ExecReload=/usr/bin/pgpool -f /etc/pgpool/pgpool.conf reload

[Install]
WantedBy=multi-user.target
```

Esse serviço vale apenas para a versão 4.3 extraída do código base em
2022-02-14, do diretório `src/redhat/pgpool.service`.

Copie o serviço para `/etc/systemd/system/pgpool.service`.
Agora faça `systemctl daemon-reload`.

### Arquivos de configuração

Todos os arquivos de configuração e scripts, estão no diretório `/etc/pgpool/`.
Como o Pgpool foi compilado, provável que não exista o diretório `/etc/pgpool`,
então é necessário criar, com `mkdir -p /etc/pgpool`.

Os arquivos de configuração são:
: - `failover.sh`
  - `pcp.conf`
  - `pgpool.conf`
  - `pool_hba.conf`
  - `pool_passwd`
