---
title: Como instalar o FreeIPA
---

O IPA é uma solução integrada de gerenciamento de informações de segurança
que combina Fedora, 389 Directory Server, MIT Kerberos, NTP e DNS.
Basicamente um frontend que une estas soluções. Ele tem uma interface de
administração via linha de comando (CLI) e uma interface web para gerenciamento de
usuários e grupos.

## Configurando a rede

É preciso garantir que o `/etc/hosts` está configurado corretamente:

```
localhost 127.0.0.1
```

## Kerberos

[Use este guia mais detalhado da instalação e configuração do Kerberos](https://web.mit.edu/kerberos/krb5-devel/doc/admin/install_kdc.html#install-and-configure-the-primary-kdc).

## Instalando pacotes do Kerberos

Atualize os pacotes:

```console
$ apt update && apt upgrade
```

Instale os pacotes do Kerberos:

```console
$ apt-get install krb5-{admin-server,kdc}
```

Você será direcionado para uma configuração interativa.

Configurando um Realm Kerberos:

```
> Enter
```

O Realm Padrão:

```
> FREEIPA.C3SL.UFPR.BR
```

Informe os hostnames dos servidores Kerberos:

```
> freeipa.c3sl.ufpr.br
```

Informe o hostname do servidor administrativo:

```
> freeipa.c3sl.ufpr.br
```

## Configurando o Kerberos

Arquivo `/etc/krb5.conf`:

```ini
[libdefaults]
    default_realm = FREEIPA.C3SL.UFPR.BR

# The following krb5.conf variables are only for MIT Kerberos.
    kdc_timesync = 1
    ccache_type = 4
    forwardable = true
    proxiable = true

# The following encryption type specification will be used by MIT Kerberos
# if uncommented.  In general, the defaults in the MIT Kerberos code are
# correct and overriding these specifications only serves to disable new
# encryption types as they are added, creating interoperability problems.
#
# The only time when you might need to uncomment these lines and change
# the enctypes is if you have local software that will break on ticket
# caches containing ticket encryption types it doesn't know about (such as
# old versions of Sun Java).

    allow_weak_crypto = false
#   default_tgs_enctypes = des3-hmac-sha1
#   default_tkt_enctypes = des3-hmac-sha1
#   permitted_enctypes = des3-hmac-sha1

# The following libdefaults parameters are only for Heimdal Kerberos.
    fcc-mit-ticketflags = true

[realms]
    FREEIPA.C3SL.UFPR.BR = {
        kdc = freeipa.c3sl.ufpr.br
        admin_server = freeipa.c3sl.ufpr.br
                default_domain = freeipa.c3sl.ufpr.br
                passwd_server = freeipa.c3sl.ufpr.br
                #database_module = openldap_ldapconf
}

[domain_realm]
    .freeipa.c3sl.ufpr.br = FREEIPA.C3SL.UFPR.BR
    freeipa.c3sl.ufpr.br = FREEIPA.C3SL.UFPR.BR

[logging]
    kdc = FILE:/var/log/krb5-kdc.log
    admin_server = FILE:/var/log/kadmin.log
    default = FILE:/var/log/krb5.log
```

Arquivo `/etc/krb5kdc/kdc.conf`:

```
[kdcdefaults]
    kdc_ports = 750,88

[realms]
    FREEIPA.C3SL.UFPR.BR = {
        database_name = /var/lib/krb5kdc/principal
        admin_keytab = FILE:/etc/krb5kdc/kadm5.keytab
        acl_file = /etc/krb5kdc/kadm5.acl
        key_stash_file = /etc/krb5kdc/stash
        kdc_ports = 750,88
        max_life = 10h 0m 0s
        max_renewable_life = 7d 0h 0m 0s
        master_key_type = aes256-cts-hmac-sha1-96
        #supported_enctypes = aes256-cts:normal aes128-cts:normal
        default_principal_flags = +preauth
    }
```

## Exportando variáveis

```console
export KRB5_CONFIG=/yourdir/krb5.conf
export KRB5_KDC_PROFILE=/yourdir/kdc.conf
```

## Criando a database do Kerberos

```console
$ kdb5_util create -r FREEIPA.C3SL.UFPR.BR -s
```

Adicione administradores ao arquivo ACL escrevendo os administradores
em `/etc/krb5kdc/kadm5.acl`:

```
*/admin@FREEIPA.C3SL.UFPR.BR    *
*/root@FREEIPA.C3SL.UFPR.BR
```

E adicione administradores à database do Kerberos:

```
shell% kadmin.local

kadmin.local: addprinc admin/admin@FREEIPA.C3SL.UFPR.BR

No policy specified for "admin/admin@FREEIPA.C3SL.UFPR.BR";
assigning "default".
Enter password for principal admin/admin@FREEIPA.C3SL.UFPR.BR:  <= Enter a password.
Re-enter password for principal admin/admin@FREEIPA.C3SL.UFPR.BR:  <= Type it again.
Principal "admin/admin@FREEIPA.C3SL.UFPR.BR" created.
kadmin.local:
```

Repetir o mesmo processo para o usuário `root`:

```
shell% kadmin.local

kadmin.local: addprinc root/admin@FREEIPA.C3SL.UFPR.BR
...
```

## Iniciando os daemons do Kerberos

```console
$ krb5kdc
$ kadmind
$ systemctl restart krb5-kdc
$ systemctl restart krb5-admin-server
```

Você pode conferir se deu certo através da listagem de unidades, filtrando por serviço:

```console
$ systemctl list-units --type=service
```

## DNS

Não é necessário o DNS estar na mesma máquina que o IPA, mas ele
deve resolver as consultas forwarding e reverse corretamente.

## Instalando as dependências

Ao tentar instalar o `freeipa-server`, existirão alguns problemas de dependência:

```
Os pacotes a seguir têm dependências desencontradas:
 freeipa-server : Depende: 389-ds-base (>= 1.4.4.16) mas não será instalado
                  Depende: chrony mas não será instalado
                  Depende: freeipa-client (= 4.9.8-1+exp1) mas 4.9.8-1~bpo11+1 está para ser instalado
                  Depende: freeipa-common (= 4.9.8-1+exp1) mas 4.9.8-1~bpo11+1 está para ser instalado
                  Depende: pki-ca (>= 10.10.6~) mas não será instalado
                  Depende: pki-kra (>= 10.10.6~) mas não será instalado
                  Depende: python3-ipaserver (= 4.9.8-1+exp1) mas não será instalado
                  Depende: libsss-certmap0 (>= 2.6.1) mas 2.4.1-2 está para ser instalado
                  Depende: libsss-nss-idmap0 (>= 2.6.1) mas 2.4.1-2 está para ser instalado
                  Depende: samba-libs (>= 2:4.13.14+dfsg) mas 2:4.13.13+dfsg-1~deb11u3 está para ser instalado
```

O FreeIPA é uma solução para o Fedora, portanto, alguns passos devem ser tomados
antes de começarmos a instalação no Debian.

## Repositórios

O arquivo `/etc/apt/sources.list` deve ter os seguintes repositórios:

```
deb http://debian.c3sl.ufpr.br/debian bullseye main
deb http://debian.c3sl.ufpr.br/debian experimental main
deb http://debian.c3sl.ufpr.br/debian unstable main
deb http://debian.c3sl.ufpr.br/debian testing main
deb http://debian.c3sl.ufpr.br/debian bullseye-backports main
deb http://debian.c3sl.ufpr.br/debian bullseye-updates main
deb http://debian.c3sl.ufpr.br/debian-security/ testing-security main
```

## Instalando pacotes

Agora que os repositórios estão atualizados, podemos instalar as seguintes dependências:

```console
$ apt update && apt upgrade
$ apt full-upgrade
```

Os pacotes a seguir têm dependências desencontradas e conflitam entre si:

```
> chrony : Conflita: time-daemon
> systemd-timesyncd : Conflita: time-daemon
```

O FreeIPA depende de mais dois pacotes:

```console
$ apt install chrony libsemanage-common freeipa-client
$ apt install freeipa-server -t experimental
```
