---
title: Como instalar o LDAP
---

## Introdução

Este howto é baseado na [documentação](https://wiki.debian.org/LDAP/OpenLDAPSetup) oficial do LDAP para o Debian:
Qualquer detalhe mais profundo pode e deve ser consultado na documentação
oficial.

Este guia passa pelos tópicos que nos são importantes da documentação
para instalar o LDAP e fazer sua integração com o Kerberos. Eventualmente,
eu explico sucintamente como funcionam alguns conceitos.

## Instalação dos pacotes

Instalamos o pacote do servidor LDAP e o pacote de seus utilitários.

```bash
apt install slapd ldap-utils
```

O prompt do debconf vai pedir uma senha para a database do administrador.

## BaseDN

Se o servidor DNS estiver configurado corretamente, o Slapd vai criar o
BaseDN usando o nome do servidor DNS.

```bash
$ ldapsearch -x -LLL -s base -b "" namingContexts
dn:
namingContexts: dc=c3sl,dc=ufpr,dc=br
```

## Utilitários

Após a instalação inicial, teremos utilitários de dois tipos:
Os `específicos` e os `genéricos`. Consulte a documentação para uma
lista completa de todos os utilitários.

### OpenLDAP específicos

Os utilitários de baixo nível, usadas somente pelo servidor, são os
iniciados por `slapd`. Podem ser executadas, inclusive, se o servidor
não estiver rodando.

ex: `slapadd`, `slapdn`...

### LDAP genéricos

Os utilitários genéricos que podem ser usados tanto pelo servidor
como por clientes e são iniciadas por `ldap`.

ex: `ldapsearch`, `ldapadd`...

## Configuração

O Slapd armazena suas configurações diretamente em sua database.

As alterações nas configuração são aplicadas sem a necessidade de
reiniciar o slapd server, pois as alterações são feitas diretamente
na database.

Na verdade, o Slapd tem várias databases, cada uma correspondente a
um número. A configuração é a database 0.

### Printar árvore de configuração

```bash
ldapsearch -LLLQ -Y EXTERNAL -H ldapi:/// -b cn=config dn
```

## Schemas

Para adicionar schemas no LDAP, usamos o utilitário `ldapadd`.
Posteriormente, adicionaremos o schema do Kerberos, por enquanto, seguimos.

## Database

O OpenLDAP oferece suporte a vários back-ends de banco de dados diferentes,
vamos utilizar o padrão o MDB, pois é rápido e cumpre nossas necessidades.

## Administrator Access

O administrador é definido separadamente na configuração do LDAP, nos atributos
olcRootDN e olcRootPW:

```bash
ldapsearch -LLLQ -Y EXTERNAL -H ldapi:/// -b "olcDatabase={1}mdb,cn=config" olcRootDN OlcRootPW
```

Observe que o olcRootDN não precisa se referir a uma entrada real no banco
de dados. Além disso, o usuário definido no olcRootDN não está sujeito a
nenhuma lista de controle de acesso, ACL, verificações ou outras verificações.

## Access Control List

As listas de controle de acesso atuais são armazenadas como atributos `olcAccess`:

```bash
ldapsearch -LLLQ -Y EXTERNAL -H ldapi:/// -b cn=config -s one olcAccess
```

O comportamento default é que apenas o usuário root pode alterar as configurações,
através de conexão SASL EXTERNAL authentication (possivel através de ldapi:///).

```bash
ldapsearch -Y EXTERNAL -H ldapi:/// -b "cn=config"
```

Todos os usuários podem fazer procuras nos diretórios, sem nenhuma autenticação.
Mas apenas o usuário root pode fazer alterações.

```bash
ldapsearch -x -b "dc=c3sl,dc=ufpr,dc=br"
```

Para fazer consultas como admin da database, use autenticação simples e use a senha
informada na instalação:

```bash
ldapsearch -x -D "cn=admin,dc=c3sl,dc=ufpr,dc=br" -W -b "dc=c3sl,dc=ufpr,dc=br"
```

Consultar a documentação para alterar o comportamento padrão, como múltiplos adms, etc.

## Basic Tasks

### Logging

Os logs são mandados para o syslog e/ou Journal. O level default do log é `none`.
Para alterar o nível de log, troque o log level para `stats`:

```bash
$ ldapmodify -Q -H ldapi:/// -Y EXTERNAL <<EOF
dn: cn=config
changetype: modify
replace: olcLogLevel
olcLogLevel: stats
EOF
modifying entry "cn=config"
```

### Tamanho da Database

O tamanho default da database é 1GB. Quando atingir o limite máximo, todas as tentativas
de escrita na database vão falhar.

A database fica armazenada em `/var/lib/ldap/data.mdb`
e podemos checar seu tamanho desta forma:

```bash
du -h /var/lib/ldap/data.mdb
```

Para checar o limite definido para nossa database:

```bash
ldapsearch -H ldapi:/// -Y EXTERNAL -b "olcDatabase={1}mdb,cn=config" olcDbMaxSize
```

Em `olcDbMaxSize`, podemos conferir o tamanho em bytes. Para alterar, por exemplo,
para 10GiB:

```bash
$ ldapmodify -H ldapi:/// -Y EXTERNAL << EOF
dn: olcDatabase={1}mdb,cn=config
changetype: modify
replace: olcDbMaxSize
olcDbMaxSize: 10737418240

EOF
```

OBS: Em sistemas 32-bits há limites para a memória máxima, consultar documentação.

### Mudando a senha de administrador

Use o `slappasswd` para gerar uma hash da senha nova e depois use ldapmodify para
atualizar a senha hasheada no atributo `olcRootPW` na configuração da database.

```bash
$ slappasswd
New password: newpassword
Re-enter new password: newpassword
{SSHA}zYHmkowzdMxwX0KtEPNak5IbzfY8YmdQ
$ ldapmodify -H ldapi:/// -Y EXTERNAL
SASL/EXTERNAL authentication started
SASL username: gidNumber=0+uidNumber=0,cn=peercred,cn=external,cn=auth
SASL SSF: 0
dn: olcDatabase={1}mdb,cn=config
changetype: modify
replace: olcRootPW
olcRootPW: {SSHA}zYHmkowzdMxwX0KtEPNak5IbzfY8YmdQ

modifying entry "olcDatabase={1}mdb,cn=config"
```

Depois, use ldappasswd para alterar a senha da conta de administrador:

```bash
$ ldappasswd -x -D cn=admin,dc=c3sl,dc=ufpr,dc=br -W -S
New password: newpassword
Re-enter new password: newpassword
Enter LDAP Password: oldpassword
```

Sim, aparentemente isto é um BUG do LDAP em que a senha precisa ser alterada duas vezes.
Portanto, é recomendado testar a autenticação com a senha antiga e a nova:

```bash
$ ldapwhoami -x -D cn=admin,dc=c3sl,dc=ufpr,dc=br -W
Enter LDAP Password: oldpassword
ldap_bind: Invalid credentials (49)

$ ldapwhoami -x -D cn=admin,dc=c3sl,dc=ufpr,dc=br -W
Enter LDAP Password: newpassword
dn:cn=admin,dc=c3sl,dc=ufpr,dc=br
```

### Indexes

É possível melhorar a performance aumentando o número de indexes.
Conferir na documentação.

### TLS/SSL

Na nossa configuração atual, não usamos TLS/SSL. Para ativar, o servidor
precisa de um certificado e uma chave privada associada, ambas em formato
PEM. Clientes precisariam de um certificado CA.

## Integração com Kerberos

A ideia é integrar o kerberos com o LDAP, utilizando o LDAP como back-end.
Configurar o Kerberos para usar o OpenLDAP como back-end permite vincular
os Principles do Kerberos às contas de usuário/máquina e pode ser usado no
lugar de senhas comuns armazenadas no atributo `userPassword`.

Na configuração atual da Urquell, o servidor LDAP e Kerberos na mesma máquina,
portanto basta utilizar o LDAP sobre ldapi:/// unix socket. Mas é possível
utilizar em máquinas distintas, configurando o LDAP sobre o TLS.

### Instalação Inicial

Instale os pacotes:

```bash
apt install krb5-kdc-ldap krb5-admin-server schema2ldif
```

E carregue o SCHEMA do Kerberos:

```bash
zcat /usr/share/doc/krb5-kdc-ldap/kerberos.openldap.ldif.gz | ldapadd -Q -Y EXTERNAL -H ldapi:///
adding new entry "cn=kerberos,cn=schema,cn=config"
```

Adicione um index em `krbPrincipalName` (melhora a performance) para as databases
onde você pretende armazenar os dados do kerberos.

```bash
$ ldapmodify -Q -Y EXTERNAL -H ldapi:/// <<EOF
dn: olcDatabase={1}mdb,cn=config
add: olcDbIndex
olcDbIndex: krbPrincipalName eq,pres,sub
EOF

modifying entry "olcDatabase={1}mdb,cn=config"
```

Em seguida, você precisa criar e configurar duas entradas que serão
usadas pelos servidores Kerberos para conectar-se ao OpenLDAP.
Se você estiver executando o Kerberos e o OpenLDAP no mesmo sistema,
essas etapas serão opcionais, mas recomendadas.

Para manter as coisas bem separadas, tudo será criado em uma unidade
organizacional(ou) separada. Note que uma ligação simples (-x -D) é
usada em vez de uma ligação externa, pois o acesso de gravação ao
DIT dc=c3sl,dc=ufpr,dc=br é necessário:

```bash
$ ldapadd -x -D cn=admin,dc=c3sl,dc=ufpr,dc=br -W <<EOF
dn: ou=Services,dc=c3sl,dc=ufpr,dc=br
objectClass: organizationalUnit
objectClass: top
ou: Services

dn: ou=kerberos,ou=Services,dc=c3sl,dc=ufpr,dc=br
objectClass: organizationalUnit
objectClass: top
ou: kerberos

dn: uid=kdc,ou=kerberos,ou=Services,dc=c3sl,dc=ufpr,dc=br
uid: kdc
objectClass: account
objectClass: simpleSecurityObject
userPassword: {CRYPT}x
description: Kerberos KDC Account

dn: uid=kadmin,ou=kerberos,ou=Services,dc=c3sl,dc=ufpr,dc=br
uid: kadmin
objectClass: account
objectClass: simpleSecurityObject
userPassword: {CRYPT}x
description: Kerberos Admin Server Account
EOF
Enter LDAP Password: SECRET

adding new entry "ou=Services,dc=c3sl,dc=ufpr,dc=br"

adding new entry "ou=kerberos,ou=Services,dc=c3sl,dc=ufpr,dc=br"

adding new entry "uid=kdc,ou=kerberos,ou=Services,dc=c3sl,dc=ufpr,dc=br"

adding new entry "uid=kadmin,ou=kerberos,ou=Services,dc=c3sl,dc=ufpr,dc=br"
```

As senhas que foram definidas para os dois usuários eram placeholders,
portanto é gere novas senhas reais:

```bash
$ ldappasswd -x -D cn=admin,dc=c3sl,dc=ufpr,dc=br -W -S uid=kdc,ou=kerberos,ou=Services,dc=c3sl,dc=ufpr,dc=br
New password: SECRET_A
Re-enter new password: SECRET_A
Enter LDAP Password: SECRET

$ ldappasswd -x -D cn=admin,dc=c3sl,dc=ufpr,dc=br -W -S uid=kadmin,ou=kerberos,ou=Services,dc=c3sl,dc=ufpr,dc=br
New password: SECRET_B
Re-enter new password: SECRET_B
Enter LDAP Password: SECRET
```

Agora as ACLs precisam ser alteradas para conceder permissões
às duas novas entidades. Algo assim:

```bash
$ ldapmodify -Q -Y EXTERNAL -H ldapi:/// <<EOF
dn: olcDatabase={1}mdb,cn=config
add: olcAccess
olcAccess: {0}to attrs=krbPrincipalKey
  by anonymous auth
  by dn.exact="uid=kdc,ou=kerberos,ou=Services,dc=c3sl,dc=ufpr,dc=br" read
  by dn.exact="uid=kadmin,ou=kerberos,ou=Services,dc=c3sl,dc=ufpr,dc=br" write
  by self write
  by * none
-
add: olcAccess
olcAccess: {1}to dn.subtree="cn=krbContainer,ou=kerberos,ou=Services,dc=c3sl,dc=ufpr,dc=br"
  by dn.exact="uid=kdc,ou=kerberos,ou=Services,dc=c3sl,dc=ufpr,dc=br" read
  by dn.exact="uid=kadmin,ou=kerberos,ou=Services,dc=c3sl,dc=ufpr,dc=br" write
  by * none
EOF

modifying entry "olcDatabase={1}mdb,cn=config"
```

Se quisermos deixar o KDC rastrear a hora do último sucesso de login
e o lockout, na linha do `uid=kdc,ou=kerberos,ou=Services...` precisamos
dar permissão de escrita.

OBS: se quisermos adicionar os PRINCIPLES do Kerberos a entradas em subárvores
diferentes de cn=krbContainer,ou=kerberos,ou=Services,dc=c3sl,dc=ufpr,dc=br,
como ou=People,dc=c3sl,dc=ufpr,dc=br, então as ACLs precisará ser adaptado para
conceder as permissões apropriadas. Acho desnecessário.

### Alterações em arquivos do Kerberos

Agora, criamos o `/etc/krb5.conf` em todos os clientes.

Em `/etc/krb5kd/kdc.conf`, podemos adicionar o seguinte módulo:

```bash
[dbmodules]
        openldap_ldapconf = {
                db_library = kldap

                ldap_kerberos_container_dn = cn=kerberos,ou=Services,dc=c3sl,dc=ufpr,dc=br

                # if either of these is false, then the ldap_kdc_dn needs to
                # have write access as explained above
                disable_last_success = true
                disable_lockout = true
                ldap_conns_per_server = 5
                ldap_servers = ldapi:///

                # this object needs to have read rights on
                # the realm container, principal container and realm sub-trees
                ldap_kdc_dn = "uid=kdc,ou=kerberos,ou=Services,dc=c3sl,dc=ufpr,dc=br"

                # this object needs to have read and write rights on
                # the realm container, principal container and realm sub-trees
                ldap_kadmind_dn = "uid=kadmin,ou=kerberos,ou=Services,dc=c3sl,dc=ufpr,dc=br"

                # this file will be used to store plaintext passwords used
                # to connect to the LDAP server
                ldap_service_password_file = /etc/krb5kdc/service.keyfile

                # OR, comment out ldap_kdc_dn, ldap_kadmind_dn and
                # ldap_service_password_file above and enable the following
                # two lines, if you skipped the step of creating entries/users
                # for the Kerberos servers

                #ldap_kdc_sasl_mech = EXTERNAL
                #ldap_kadmind_sasl_mech = EXTERNAL
                #ldap_servers = ldapi:///
        }
```

Agora, use `kdb5_ldap_util` para criar o domínio Kerberos configurando
algumas entradas básicas no banco de dados LDAP:

```bash
$ kdb5_ldap_util -D cn=admin,dc=c3sl,dc=ufpr,dc=br create -subtrees dc=c3sl,dc=ufpr,dc=br -r EXAMPLE.COM -s -H ldapi:///
Password for "cn=admin,dc=c3sl,dc=ufpr,dc=br":
Initializing database for realm 'EXAMPLE.COM'
You will be prompted for the database Master Password.
It is important that you NOT FORGET this password.
Enter KDC database master key: SOME_SECRET
Re-enter KDC database master key to verify: SOME_SECRET
```

E use o mesmo utilitário para armazenar as senhas LDAP que kadmin e
kdc precisam conectar ao servidor LDAP:

```bash
$ kdb5_ldap_util -D cn=admin,dc=c3sl,dc=ufpr,dc=br stashsrvpw -f /etc/krb5kdc/service.keyfile uid=kdc,ou=kerberos,ou=Services,dc=c3sl,dc=ufpr,dc=br
Password for "cn=admin,dc=c3sl,dc=ufpr,dc=br": SECRET
Password for "uid=kdc,ou=kerberos,ou=Services,dc=c3sl,dc=ufpr,dc=br": SECRET_A
Re-enter password for "uid=kdc,ou=kerberos,ou=Services,dc=c3sl,dc=ufpr,dc=br": SECRET_A
$ kdb5_ldap_util -D cn=admin,dc=c3sl,dc=ufpr,dc=br stashsrvpw -f /etc/krb5kdc/service.keyfile uid=kadmin,ou=kerberos,ou=Services,dc=c3sl,dc=ufpr,dc=br
Password for "cn=admin,dc=c3sl,dc=ufpr,dc=br": SECRET
Password for "uid=kadmin,ou=kerberos,ou=Services,dc=c3sl,dc=ufpr,dc=br": SECRET_B
Re-enter password for "uid=kadmin,ou=kerberos,ou=Services,dc=c3sl,dc=ufpr,dc=br": SECRET_B
```

Agora, já podemos iniciar o servidor krb5-admin-server:

```bash
systemctl start krb5-kdc krb5-admin-server
```

Mude as permissões do keytab para que o usuário ou grupo
openldap possa lê-lo, mas mais ninguém:

```bash
chown root:openldap /etc/krb5.ldap.keytab
chmod 0640 /etc/krb5.ldap.keytab
```

Em seguida, edite /etc/default/slapd para informar ao slapd onde encontrar o
keytab descomentando a exportação `KRB5_KTNAME` e configurando-a para o local
do keytab que você acabou de criar:

```bash
export KRB5_KTNAME=/etc/krb5.ldap.keytab
```

Reinicie o slapd:

```bash
systemctl restart slapd
```

Em seguida, crie uma expressão regular mapeando os principles do
Kerberos para entradas LDAP, algo como:

```bash
$ ldapmodify -H ldapi:/// -Y EXTERNAL << EOF
dn: cn=config
changetype: modify
replace: olcAuthzRegexp
olcAuthzRegexp: {0}"uid=([^/]*)/admin,(cn=example.com,)?cn=gssapi,cn=auth" "cn=admin,dc=c3sl,dc=ufpr,dc=br"
olcAuthzRegexp: {1}"uid=([^/]*),(cn=example.com,)?cn=gssapi,cn=auth" "uid=$1,ou=People,dc=c3sl,dc=ufpr,dc=br
EOF
```

### Testando

Inicialize um principal Kerberos e tente conectar-se a ele (usando o mecanismo -Y GSSAPI):

```bash
$ kinit alice
Password for alice@EXAMPLE.COM:

$ klist
Ticket cache: FILE:/tmp/krb5cc_xyz
Default principal: alice@EXAMPLE.COM
...

ldapwhoami -Q -Y GSSAPI -H ldapi:///
dn:uid=alice,ou=People,dc=c3sl,dc=ufpr,dc=br
```

## Backup

Quase sempre negligenciado, mas extremamente importante, precisamos
falar sobre o Backup da base de dados. Podemos, de maneira simples,
fazer o backup da base de dados do LDAP.

Para fazer backup de sua configuração e bancos de dados LDAP, usamos
o utilitáiro `slapcat`, que gera DITs inteiros no formato LDIF e que
pode ser executado enquanto o slapd está em execução:

```bash
slapcat -n0 -l backup_config.ldif
```

O comando acima cria um backup do banco de dados 0
(que é o banco de dados cn=config) e o grava em `backup_config.ldif`.

Após o backup da configuração, você precisa repetir o comando para cada um
dos diretórios de dados usando o número do banco de dados correspondente:

```bash
slapcat -n1 -l backup_data.ldif
```

É recomendado deixar a database em read-only antes de fazer o backup,
para uma maior segurança:

```bash
$ ldapmodify -Q -Y EXTERNAL -H ldapi:/// <<EOF
dn: olcDatabase={1}mdb,cn=config
changeType: modify
replace: olcReadOnly
olcReadOnly: TRUE
EOF
modifying entry "olcDatabase={1}mdb,cn=config"
```

Após a conclusão do backup, você pode desativar o modo somente
leitura da seguinte maneira:

```bash
$ ldapmodify -Q -Y EXTERNAL -H ldapi:/// <<EOF
dn: olcDatabase={1}mdb,cn=config
changeType: modify
replace: olcReadOnly
olcReadOnly: FALSE
EOF
modifying entry "olcDatabase={1}mdb,cn=config"
```

OBS: não coloque o banco de dados de configuração (olcDatabase={0}config,cn=config)
no modo somente leitura, pois a desativação do modo somente leitura requer gravação
no banco de dados de configuração, horácio na certa.
