---
title: Como recuperar discos de raiz falhos
---

Geralmente as máquinas implementam pelo menos RAID1 (espelhamento) nos seus
discos de raiz para permitir que mesmo que falhe um disco, o sistema continue
funcionando normalmente. Porém, pode acontecer, devido a fatores como os discos
serem colocados em tempos semelhantes e serem do mesmo lote, que os dois discos
da raiz falhem simultaneamente. Para isso, precisamos recorrer então ao
[backup da duvel](/docs/services/duvel-backup) (já testou seu backup hoje?):

1. Pegue um pendrive bootável com SSH (de preferência que já tenha a sua chave SSH)
, rede e utilidades já instaladas. É possível encontrar uma imagem Arch Linux
para pendrive na [](/docs/machines/virtuals/abbey) em `/root/ramfs/isos/`. É bom manter
no data center uns dois pendrives assim.
1. Vá ao data center e coloque o pendrive na máquina que falhou as duas raizes.
1. Coloque nessa máquina dois SSDs de **mesmo tamanho** onde estavam as duas raízes.
1. Ligue a máquina e faça boot no pendrive via IPMI.
1. Particione cada um dos SSDs sendo:
   * A primeira partição com 1MB no formato BIOS (não usamos UEFI para não precisar
   criar mais partições).
   * A segunda partição com o restante do espaço para o MD.

   Isso pode ser feito usando o `sgdisk` da seguinte maneira:

   ```bash
   for DISK in $DISK1 $DISK2; do
      # limpa tabelas de partição
      sgdisk --zap-all $DISK
      # adiciona partição de boot BIOS modo legado (ef02 BIOS boot partition)
      sgdisk --new 0:0:+1M --typecode 0:ef02 --change-name 0:BIOS  $DISK
      # adiciona partição para o MD (fd00 Linux RAID)
      sgdisk --new 0:0:0   --typecode 0:fd00 --change-name 0:LINUX $DISK
   done
   ```

   Você pode verificar as partições do disco rodando:

   ```console
   $ sgdisk -p $DISK1
   Disk $DISK1: 1048576 sectors, 512.0 MiB
   Sector size (logical): 512 bytes
   Disk identifier (GUID): B6C9F315-3FFA-4CFA-9A2F-E6FB7001E366
   Partition table holds up to 128 entries
   Main partition table begins at sector 2 and ends at sector 33
   First usable sector is 34, last usable sector is 1048542
   Partitions will be aligned on 2048-sector boundaries
   Total free space is 2014 sectors (1007.0 KiB)

   Number  Start (sector)    End (sector)  Size       Code  Name
      1            2048            4095   1024.0 KiB  EF02  BIOS
      2            4096         1048542   510.0 MiB   FD00  LINUX
   ```
1. Agora podemos configurar o RAID1 nas duas partições grandes.

   ```console
   $ mdadm --create /dev/md/boot --level 1 --raid-devices 2 ${DISK1}1 ${DISK2}1
   ```

1. Formate o MD com o sistema de arquivos. Para SSD, é recomendável
   utilizar o F2FS, pois é específico para SSDs e faz durar mais. A outra opção
   é ext4. Faça:

   ```console
   $ mkfs.f2fs /dev/md/boot
   ```

1. Monte o sistema de arquivos:

   ```console
   $ mkdir -p /mnt
   $ mount /dev/md/boot /mnt
   ```

1. Agora faça `rsync` do último backup da máquina direto para `/mnt`:

   ```console
   $ rsync --archive --human-readable --hard-links --xattrs --verbose --info=progress2 \
       duvel:/backup/$MACHINE/raiz/latest/ /mnt/
   ```

1. Agora podemos configurar o GRUB:

   ```console
   $ systemd-nspawn --quiet --resolve-conf replace-host --directory /mnt
   root@mnt:~# apt update
   root@mnt:~# apt dist-upgrade
   root@mnt:~# apt install grub-pc
   root@mnt:~# dpkg --configure --pending
   root@mnt:~# grub-install --target i386-pc $DISK1
   root@mnt:~# grub-install --target i386-pc $DISK2
   root@mnt:~# exit
   $ sync
   $ reboot
   ```

1. Inicie a máquina sem pendrive. Certifique-se que ela boota normalmente. Se
   não estiver bootando, se vire para consertar.
