---
title: Como usar o Kerberos Distribuído
---

## Introdução

O objetivo deste guia é simular um ambiente de autenticação através Kerberos
com LDAP de backend. Este guia é baseado na documentação oficial do MIT, wiki
do ArchLinux e um guia do Ubuntu. As referências estão no final do documento.

## Arquitetura

Utilizamos duas máquinas virtuais com imagem Debian Bookworm (v.12), uma para ser o servidor primário e outra para ser o servidor réplica.

vms:
- dodrio -- servidor primário kerberos e servidor LDAP
- ditto  -- servidor réplica kerberos

Perceba que se for reproduzido em um sistema diferente do Debian, o diretório padrão dos arquivos vai ser alterado e o nome dos pacotes.

## Kerberos

### Distribuição do serviço

Uma instalação ideal do kerberos é distribuída. Assim, caso o servidor primário não esteja disponível, um servidor réplica pode assumir e manter a disponibilidade do sistema de autenticação.

É importante ressaltar que, no kerberos, cada servidor contém uma cópia da database mas apenas o servidor primário pode escrever nela. Então, os servidores secundários apenas auxiliam na distribuição de tickets de autenticação. Mas queremos que um servidor réplica possa rapidamente se tornar o principal em caso de eventuais manutenções, quedas, ataques e/ou falhas.

### Nomes e Conceitos

Neste guia, utilizaremos os seguintes nomes:

- dodrio.c3sl.ufpr.br -- servidor primário
- ditto.c3sl.ufpr.br -- servidor réplica
- C3SL.UFPR.BR -- realm name
- .k5.C3SL.UFPR.BR - stash file
- admin/admin - admin principal

Na hora de reproduzir este guia, ajuste os nomes conforme necessário.

Alguns conceitos importantes:

- principal: Principals são usuários, hosts ou serviços do kerberos. Um principal Kerberos possui três componentes:
`primary / instance @ REALM`
- Para usuários principais, o principal é seu nome de usuário e a instância é omitida ou é uma função (por exemplo, "admin"): `myuser@EXAMPLE.COM` ou `myuser/admin@EXAMPLE.COM`.
- Para hosts, o primário é "host" e a instância é o FQDN do servidor: `host/myserver.example.com@EXAMPLE.COM`.
- Para serviços, o principal é a abreviatura do serviço e a instância é o FQDN: `nfs/myserver.example.com@EXAMPLE.COM`. O domínio muitas vezes pode ser omitido; o domínio padrão do computador local geralmente é assumido.

### Instalando e configurando o KDC primário

Na `dodrio`, instalamos os pacotes necessários para o servidor adminstrativo do Kerberos (krb5-admin-server) e o distribuidor de chaves do kerberos.

```bash
DEBIAN_FRONTEND=noninteractive # Não gera um prompt interativo
apt-get install krb5-{admin-server,kdc}
```

### Arquivos de configuração

- `krb5.conf`: arquivo de configurações do kerberos. Contém a localização dos KDCs e permite a que os servidores réplica com o kadmin server. O caminho padrão no debian é `/etc/krb5.conf`.

```bash
[libdefaults]
  default_realm = C3SL.UFPR.BR
  allow_weak_crypto = false
  kdc_timesync = 1
  ccache_type = 4
  forwardable = true
  proxiable = true
  rdns = false
  permitted_enctypes = aes256-cts-hmac-sha1-96 aes128-cts-hmac-sha1-96

[realms]
	C3SL.UFPR.BR = {
    primary_kdc = dodrio.c3sl.ufpr.br
    admin_server = dodrio.c3sl.ufpr.br
    kdc = dodrio.c3sl.ufpr.br
    kdc = ditto.c3sl.ufpr.br
    database_module = openldap_ldapconf
    master_key_type = aes256-cts
    supported_enctypes = aes256-cts-hmac-sha1-96:normal aes128-cts-hmac-sha1-96:normal
}

[domain_realm]
  .dodrio.c3sl.ufpr.br = C3SL.UFPR.BR
  dodrio.c3sl.ufpr.br = C3SL.UFPR.BR

[login]
  krb4_convert = true
  krb4_get_tickets = false
```

**NOTA:** se estiver reproduzindo este howto, vale a pena checar se os tipos de encriptação disponíveis são atuais e bons o suficiente. Isto é extremamente importante pois se não as chaves não serão criadas e/ou você terá problemas para replicar a database. Neste momento, 2024, usamos estes tipos:

```bash
permitted_enctypes = aes256-cts-hmac-sha1-96 aes128-cts-hmac-sha1-96 # Todos os clientes Kerberos
master_key_type = aes256-cts # Servidor Kerberos
supported_enctypes = aes256-cts-hmac-sha1-96:normal aes128-cts-hmac-sha1-96:normal # Servidor Kerberos
```

- `kdc.conf`: O arquivo kdc.conf complementa krb5.conf para programas que normalmente são usados ​​apenas em um KDC, como os daemons krb5kdc e kadmind e o programa kdb5_util. Você pode mudar as portas default, caminho padrão de arquivos como a stash, log, etc. O caminho default do `kdc.conf` é `/etc/krb5kdc/kdc.conf`.

```bash
[kdcdefaults]
    kdc_ports = 750,88

[realms]
    C3SL.UFPR.BR = {
        kadmind_port = 749
        acl_file = /etc/krb5kdc/kadm5.acl
        key_stash_file = /etc/krb5kdc/stash
        max_life = 12h 0m 0s
        max_renewable_life = 7d 0h 0m 0s
        master_key_type = aes256-cts
        supported_enctypes = aes256-cts:normal aes128-cts:normal
        default_principal_flags = +preauth
        #iprop_enable = true
    }
```

### Criando a database do KDC

Iremos usar o comando `kdb5_util` para criar a database do kerberos e o arquivo stash. Um prompt será invocado para que você digite e confirme a senha da database.
```bash
kdb5_util create -r C3SL.UFPR.BR -s

Initializing database '/usr/local/var/krb5kdc/principal' for realm 'C3SL.UFPR.BR',
master key name 'K/M@C3SL.UFPR.BR'
You will be prompted for the database Master Password.
It is important that you NOT FORGET this password.
Enter KDC database master key:  <= Type the master password.
Re-enter KDC database master key to verify:  <= Type it again.
shell%
```

Isso criará cinco arquivos:
-   dois aqruivos de database do Kerberos: `principal` e `principal.ok` 
-   a database administrativa do Kerberos: `principal.kadm5`
-   o arquivo de lock da database administrativa: `principal.kadm5.lock`
-   o arquivo de stash: `stash`

No Debian, estes arquivos são gerados usualmente em `/var/lib/krb5kdc/`, a não ser que você especifique um caminho diferente no `kdc.conf`. No nosso caso, `stash` foi criado em `/etc/krb5kdc/stash`.

### Colocando administradores na ACL

Precisamos criar um arquivo de controle de acesso. O Path deste arquivo também pode ser especificado no `krb5kdc.conf`. Em nosso caso, `/etc/krb5kdc/kadm5.acl`. O arquivo segue a seguinte sintaxe:
```bash
principal  permissions  [target_principal  [restrictions]
```
Nosso arquivo:
```bash
*/admin@C3SL.UFPR.BR    *
*/root@C3SL.UFPR.BR	 *
```
O que significa que qualquer principal em C3SL.UFPR.BR com uma instância de admin ou root tem todos os privilégios administrativos. Veja [kadm5.acl](https://web.mit.edu/kerberos/krb5-latest/doc/admin/conf_files/kadm5_acl.html) para mais detalhes.

### Adicionando administradores à database do Kerberos

Agora, utilize a local admintrative tool do kerberos para adicionar os principals administrativos à database. Perceba, a razão de não utilizarmos a `kadmin` e sim a `kadmin.local` é que a `kadmin` utiliza autenticação e os administradores ainda não estão na database para serem autenticados.
```bash
kadmin.local

Authenticating as principal root/admin@EXAMPLE.COM with password.
kadmin.local:
```
Você entrará na interface interativa do `kadmin.local`. Através do comando `addprinc`, adicione os principal `admin/admin`. Entre com a senha e confirme.
```bash
addprinc admin/admin@C3SL.UFPR.BR

WARNING: no policy specified for admin/admin@EXAMPLE.COM; defaulting to no policy
Enter password for principal "admin/admin@EXAMPLE.COM": 
Re-enter password for principal "admin/admin@EXAMPLE.COM": 
Principal "admin/admin@EXAMPLE.COM" created.
```

Faça o mesmo para o principal `root/admin` e saia do `kadmin.local`.
```bash
addprinc root/admin
# ... output omitido ...
quit
```

### Iniciando o daemon do Kerberos no KDC Principal

> No servidor primário, use os seguintes comandos na shell:
```bash
krb5kdc
kadmind
```

### Testando Ticket

Podemos testar o novo principal administrativo através do comando `kinit` e `klist`:
> No servidor primário, use os seguintes comandos na shell:
```bash
kinit
Password for root@C3SL.UFPR.BR: 

klist

Ticket cache: FILE:/tmp/krb5cc_0
Default principal: root@C3SL.UFPR.BR
Valid starting     Expires            Service principal
05/08/24 14:22:08  05/09/24 00:22:08  krbtgt/C3SL.UFPR.BR@C3SL.UFPR.BR
	renew until 05/09/24 14:22:06
```

### Sincronia de relógio

Por fim, instale o serviço do systemd que garante a sincronia dos relógios para evitar bugs no kerberos:
```bash
apt install systemd-timesyncd
```

### Configurando Servidor Réplica

Vamos configurar o servidor réplica de forma que ele possa se tornar o servidor primário.
No servidor secundário, instale os pacotes do Kerberos e o de sincronia de relógio do systemd:

```bash
DEBIAN_FRONTEND=noninteractive
apt-get install krb5-{admin-server,kdc} systemd-timesyncd 
```

### Configurando as réplicas

Copie os seguintes arquivos e deixa nos respectivos diretórios:
- `/etc/krb5.conf`
- `/etc/krb5kdc/krb5kdc.conf`
- `/etc/krb5kdc/kadm5.acl`
- `/etc/krb5kdc/stash`

Crie um arquivo para controlar a propagação da database em `/etc/krb5kdc/kpropd.acl`:
> arquivo kpropd.acl No servidor réplica
```bash
host/dodrio.c3sl.ufpr.br@C3SL.UFPR.BR
host/ditto.c3sl.ufpr.br@C3SL.UFPR.BR
```

### Criando keytabs

Todo KDC precisa de uma host key na database do kerberos. Elas são utilizadas para autenticação na propagação da database. No servidor primário, abra a interface administrativa e adicione os host do servidor primário e os réplicas:


> No servidor primário, acesse o kadmin e o comando addprinc:
```bash
kadmin
kadmin: addprinc -randkey host/dodrio.c3sl.ufpr.br
No policy specified for "host/dodrio.c3sl.ufpr.br@C3SL.UFPR.BR"; assigning "default"
Principal "host/dodrio.c3sl.ufpr.br@C3SL.UFPR.BR" created.

# Ainda no Servidor primário
kadmin: addprinc -randkey host/ditto.c3sl.ufpr.br
No policy specified for "host/ditto.c3sl.ufpr.br@C3SL.UFPR.BR"; assigning "default"
Principal "host/ditto.c3sl.ufpr.br@C3SL.UFPR.BR" created.
```

Agora, cada host deverá extrair a sua própria chave.

> No servidor primário, acesse o kadmin e o comando ktadd:
```bash
kadmin: ktadd host/dodrio.c3sl.ufpr.br
Entry for principal host/dodrio.c3sl.ufpr.br with kvno 2, encryption
    type aes256-cts-hmac-sha1-96 added to keytab FILE:/etc/krb5.keytab.
Entry for principal host/dodrio.c3sl.ufpr.br with kvno 2, encryption
    type aes128-cts-hmac-sha1-96 added to keytab FILE:/etc/krb5.keytab.
Entry for principal host/dodrio.c3sl.ufpr.br with kvno 2, encryption
    type aes256-cts-hmac-sha384-192 added to keytab FILE:/etc/krb5.keytab.
Entry for principal host/dodrio.c3sl.ufpr.br with kvno 2, encryption
    type arcfour-hmac added to keytab FILE:/etc/krb5.keytab.
```

> No servidor secundário, acesse o kadmin e o comando ktadd:
```bash
kadmin: ktadd host/ditto.c3sl.ufpr.br
Entry for principal host/ditto.c3sl.ufpr.br with kvno 2, encryption
    type aes256-cts-hmac-sha1-96 added to keytab FILE:/etc/krb5.keytab.
Entry for principal host/ditto.c3sl.ufpr.br with kvno 2, encryption
    type aes128-cts-hmac-sha1-96 added to keytab FILE:/etc/krb5.keytab.
Entry for principal host/ditto.c3sl.ufpr.br with kvno 2, encryption
    type aes256-cts-hmac-sha384-192 added to keytab FILE:/etc/krb5.keytab.
Entry for principal host/ditto.c3sl.ufpr.br with kvno 2, encryption
    type arcfour-hmac added to keytab FILE:/etc/krb5.keytab.
```

Você pode conferir, em cada um dos hosts, se deu certo através do comando `klist`:

> No servidor primário, digite:
```bash
dodrio:~# klist -k
Keytab name: FILE:/etc/krb5.keytab
KVNO Principal
---- --------------------------------------------------------------------------
   2 host/dodrio.c3sl.ufpr.br@C3SL.UFPR.BR
   2 host/dodrio.c3sl.ufpr.br@C3SL.UFPR.BR
```

> No servidor secundário, digite:
```bash
ditto:~# klist -k
Keytab name: FILE:/etc/krb5.keytab
KVNO Principal
---- --------------------------------------------------------------------------
   3 host/ditto.c3sl.ufpr.br@C3SL.UFPR.BR
   3 host/ditto.c3sl.ufpr.br@C3SL.UFPR.BR
```

Caso algum erro seja cometido, [remova as chaves](https://web.mit.edu/kerberos/krb5-latest/doc/admin/admin_commands/kadmin_local.html#ktremove) e recomece.


### Propagando a database

No **servidor secundário**, instale o `kpropd` daemon, que vai escutar tentativas de comunicação do `kprop` do servidor primário:

```
apt install krb5-kpropd
```

O serviço já irá funcionar. Você pode conferir da seguinte forma:
```bash
systemctl status krb5-kpropd
● krb5-kpropd.service - Kerberos 5 slave KDC update server
     Loaded: loaded (/lib/systemd/system/krb5-kpropd.service; enabled; preset: enabled)
     Active: active (running) since Mon 2024-05-20 10:49:20 -03; 1 week 6 days ago
# ..
```

No **servidor primário**, faça o dump da database e chame o comando da réplica para o KDC secundário:
```
kdb5_util dump /var/lib/krb5kdc/dump
kprop -r C3SL.UFPR.BR -f /var/lib/krb5kdc/dump ditto.c3sl.ufpr.br
```
Há um script python que automatiza esta rotina, visto que deverá ser feita com frequência.

Por fim, inicie o KDC secundário:
```bash
krb5kdc
```

## LDAP

Para integrar o Kerberos com o LDAP, acesse a [parte 2](https://gitlab.c3sl.ufpr.br/c3root/docs/-/blob/main/source/pages/guides/ldap.md) deste guia.

## Referências

- [Guia de instalação - MIT Kerberos Documentation:](https://web.mit.edu/kerberos/krb5-latest/doc/admin/install.html) guia mais detalhado.
- [Kerberos - ArchWiki:](https://wiki.archlinux.org/title/Kerberos#Service_principals_and_keytabs) guia com alguns exemplos interessantes e detalhes do NFS.
- [How to installl a Kerberos Server:](https://ubuntu.com/server/docs/how-to-install-a-kerberos-server) guia mais objetivo.
