---
title: Como fazer o deploy de um cluster Kubernetes com TalosLinux
tags: [kubernetes, talos-linux]
---

## Faça o deploy das VMs com OpenTofu

Veja `https://gitlab.c3sl.ufpr.br/c3root/k8s/terraform` para ter uma base.

Depois de criadas as VMs, aloque os IPs usando o `netboxadm`:

```sh
ssh netbox
cd netboxadm
for i in {1..3}; do ./netboxadm.py vm "ctl${i}-k8stest" alloc-ip; done
for i in {1..3}; do ./netboxadm.py vm "wrk${i}-k8stest" alloc-ip; done
./netboxadm.py update dhcp
./netboxadm.py update dns
```

Revise e aceite as mudanças do merge request criado no gitlab.

Caso a máquina não receba IP mesmo tendo dado certo a pipeline e o IP esteja
aparecendo no Netbox, veja o lease do IP na estrella.

## Gere o arquivo de configuração do Talos

```sh
talosctl gen config \
    $CLUSTER_NAME \
    https://$CONTROL_PLANE_IP:6443 \
    --install-image factory.talos.dev/installer/ce4c980550dd2ab1b17bbf2b08801c7eb59418eafe8f279833297925d67c7515:v1.9.2
```

Isso gera a configuração do `controlplane.yaml`, `worker.yaml` e `talosconfig`
usando a imagem que já tem instalado o QEMU Quest Agent.

## Configurando o control plane

```yaml
# controlplane.yaml
cluster:
  network:
    cni:
      name: none
  proxy:
    disabled: true
```

Isso remove o `kube-proxy` e o `kube-flannel`. Depois vai ser instalado
`cilium`, que substitui esses componentes.

Aplique a mesma configuração em todos os control planes:

```sh
talosctl apply-config --insecure --nodes $CONTROL_PLANE_IP --file controlplane.yaml
```

## Configurando um endpoint com DNS

Crie no netbox 3 records com o seguinte formato:

```
k8stest.c3sl.ufpr.br A 200.236.31.205
                     A 200.236.31.204
                     A 200.236.31.203
```

`k8stest.c3sl.ufpr.br` vai ser um endpoint que vai servir como load-balancer
entre os control-planes.

## Configurando os workers

Faça a mesma mudança no yaml do worker para retirar o `kube-proxy` e o
`kube-flannel`.

Além disso, configure o worker para usar o endpoint criado no passo anterior:

```yaml
# worker.yaml
cluster:
    controlPlane:
        endpoint: https://k8stest.c3sl.ufpr.br:6443
```

```sh
talosctl apply-config --insecure --nodes $WORKER_IP --file worker.yaml
```

## Bootstrap do etcd

```sh
export TALOSCONFIG="$(pwd)/talosconfig"
talosctl config endpoint $CONTROL_PLANE_IP
talosctl config node $CONTROL_PLANE_IP

talosctl bootstrap
```

## Conseguindo acesso ao cluster k8s

```sh
talosctl kubeconfig .
```

## Instalação temporária do cilium

```sh
cilium install \
    --set ipam.mode=kubernetes \
    --set kubeProxyReplacement=true \
    --set securityContext.capabilities.ciliumAgent="{CHOWN,KILL,NET_ADMIN,NET_RAW,IPC_LOCK,SYS_ADMIN,SYS_RESOURCE,DAC_OVERRIDE,FOWNER,SETGID,SETUID}" \
    --set securityContext.capabilities.cleanCiliumState="{NET_ADMIN,SYS_ADMIN,SYS_RESOURCE}" \
    --set cgroup.autoMount.enabled=false \
    --set cgroup.hostRoot=/sys/fs/cgroup \
    --set k8sServiceHost=localhost \
    --set k8sServicePort=7445
```

## Bootstrap do fluxcd

```sh
flux bootstrap gitlab \
  --owner=c3root/k8s \
  --repository=fluxcd \
  --hostname=gitlab.c3sl.ufpr.br \
  --path=clusters/test \
  --token-auth
```

```sh
cilium uninstall
flux reconcile source git flux-system
```
