---
title: Política de gerenciamento do RAID
---

O RAID permite redundância de dados, mas traz consigo sérios problemas que
devem ser gerenciados pelos administradores.

Como regra geral, RAID em hardware é uma ideia abandonada. Essa opção é
menos flexível e ficamos mais sujeitos aos frequentes problemas encontrados
e a falta de padronização presente nas implementações em hardware.
Note, porém, que teoricamente RAID em hardware é mais eficiente, pois as placas são
otimizadas para receber um comando de leitura/escrita do computador e
gerenciar o envio destes comandos de forma distribuída para todos os discos.
Quando o RAID é feito por software, a controladora de discos tem que fazer essa
mesma tarefa de distribuir comandos enviados pelo computador para todos os
discos, algo que ela não é feita para fazer da forma mais eficiente possível.

Além disso, em alguns casos o Ceph é utilizado para a mesma tarefa de garantir
a redundância de dados, que por princípio manda comandos separados para disco.
Assim, também não é benéfico comprar hardware que vai contra esse caso
de uso que vem sendo cada vez mais utilizado. As placas boas de RAID permitem
acesso direto aos discos, porém exigem hardware extra de switch de disco.

O RAID em hardware faz verificação contínua dos discos e consertam os problemas
logo que aparecem, e as placas (com bateria) fazem journal dos comandos e dados.
Ao fazer RAID em software precisamos configurar essas funcionalidades:
é necessário manter um SSD para journal, além de agendar as verificações e
correções preventivas.

A solução para RAID em software no Linux é o [](/pages/concepts/md). Porém, é
necessário lembrar que mesmo fazendo RAID em software, estamos sujeitos a falhas
presentes nas controladoras de discos e a possíveis falhas na implementação
em software (que não são incomuns).

- A verificação automática periódica é configurada através de um crontab,
  em `/etc/cron.d/mdadm`. As máquinas [](/pages/machines/base/duvel),
  [](/pages/machines/base/urquell), [](/pages/machines/base/sagres) estão configuradas para
  rodar a verificação às 7h30 do primeiro sábado do mês.
  A máquina [](/pages/machines/base/roble) está configurada para rodar a verificação
  às 0h02 do primeiro sábado do mês. Note que reparar as inconsistências
  não é a configuração padrão e precisa ser ativada quando necessário.

- A avaliação dos estados dos erros pode ser verificada através do comando:

  ```console
  $ tail /sys/block/md*/md/mismatch_cnt
  ```

  E a reparação feita através do comando:

  ```console
  $ echo repair > /sys/block/md127/md/sync_action
  ```

- Notoriamente, a [](/pages/machines/base/urquell) tem uma quantidade elevada de
  erros de paridade. Como foi feita uma troca total dos discos da máquina,
  suspeita-se que é resultado de uma falha na controladora de discos ou no
  [](/pages/concepts/md).

- Podemos acompanhar o desempenho do RAID pelo `/proc/mdstat`:

  ```console
  root@abbey:~# cat /proc/mdstat
  Personalities : [raid10] [raid1] [linear] [multipath] [raid0] [raid6] [raid5] [raid4]
  md127 : active raid6 sdh[5] sdg[4] sdd[3] sdc[2] sdb[1] sda[0]
        31255576576 blocks super 1.2 level 6, 512k chunk, algorithm 2 [6/6] [UUUUUU]
        [>....................]  resync =  0.5% (40998628/7813894144) finish=1403.7min speed=92289K/sec
        bitmap: 0/59 pages [0KB], 65536KB chunk

  md0 : active raid1 sde2[0] sdf2[1]
        2929928640 blocks super 1.2 [2/2] [UU]
        bitmap: 6/22 pages [24KB], 65536KB chunk

  unused devices: <none>
  ```

  E existe como limitar a velocidade máxima de resync através do comando:

  ```console
  echo 800000 > /proc/sys/dev/raid/speed_limit_max
  ```

  O padrão é 200000, e dependendo do RAID, vale a pela aumentar ou reduzir
  a velocidade máxima. Por exemplo na [](/pages/machines/virtuals/abbey), esse
  número é quatro vezes maior, pois não é muito utilizado. Na [](/pages/machines/cluster/c3hpc),
  foi reduzido para 50000 para não afetar o desempenho do `/home/` do cluster,
  algo que provavelmente foi feito também na [](/pages/machines/base/urquell).
