---
title: PM002
description: Gerenciar a rede usando NetBox.
---

A rede do Departamento de Informática é complexa. Contamos hoje com por volta de
56 VLANs e prefixos de IP correspondentes, além de por volta de 400 máquinas físicas
e 125 máquinas virtuais. Tradicionalmente esse sistema foi gerenciado através
de conjuntos de arquivos texto localizado nas máquinas responsáveis pelos serviços
correspondentes junto de scripts que criavam as configurações das máquinas a
partir desses arquivos texto. Porém com a necessidade de se desacoplar e
distribuir serviços em máquinas diferentes, surgem algumas problemas:

- Os arquivos texto precisam ser entendíveis pelas máquinas para gerar as
configurações mas também alteráveis por seres humanos; o resultado é que erros
de sintaxe ou erros semânticos desses arquivos geram falhas catastróficas.
- A responsabilidade de evitar que falhas catastróficas como duplicação de nomes,
IPs, MACs e outros identificadores únicos cai sobre os scripts ou do
administrador que não tem absoluta ciência de todos os componentes da rede.
- Informações sobre a mesma coisa ficam distribuídas entre as máquinas que
gerenciam serviços diferentes; o serviço de virtualização, DHCP, DNS, firewall,
backup e certificados SSL por exemplo precisam concordar ao menos na existência
de um mesmo conjunto de máquinas virtuais. A responsabilidade de manter todos
os serviços cientes da mesma informação cai sobre o administrador, que é falho.
- Quando é necessário saber alguma informação sobre a descrição e usuário
responsável por um certo item da rede, é necessário vasculhar os diversos
arquivos de texto em busca de informação e no pior caso recorrer ao e-mail
para ver se alguém sabe.
- Para mexer nos arquivos texto, é necessário acesso às máquinas correspondentes
e implica necessariamente acesso total. Não é possível ter nenhum tipo de controle
de acesso nesse modelo, ou o acesso é total ou é inexistente.
- Os arquivos texto não possuem controle de versão, tampouco controle de quem
realizou as mudanças e não é possível saber exatamente quais serão os efeitos
das mudanças a serem realizadas de antemão.

Problemas resultado desse modelo de gerência acabam sendo:
- Queda de serviços inesperada devido a problemas de sintaxe residuais
de outra pessoa que esqueceu de rodar o script (Horácio "VLAN/DHCP")
- Assinalamento de valores fora dos intervalos permitidos (Horácio "10.254.300.\*")
- Descrição das máquinas inexistente (Horácio "Esqueceram de mim 2" e "Observatório")
- Duplicação de nomes na rede (Horácio "DNS SPOOFING DETECTED!")
- Duplicação de IPs na rede (Horácio "Redundância de roteamento", "Eu não aprendo com o erro alheio..." e "IPv4 na faixa de unknown-clients na XHIPES")
- Gestão falha de backups e certificados SSL

Vamos então focar nos pontos chave que precisam ser atendidos pela solução que
desejamos usar:

- Apresentação das informações é clara e 100% pesquisável
- Consumível por máquinas para gerar arquivos de configuração
- Integridade e consistência dos dados
- Concentrar todas as informações da rede, sem buracos
- Controle de versão das alterações feitas e preferencialmente por quem, quando
e talvez até por qual razão
- Etapas podem ser puladas em casos de emergência
- É fácil reverter quando acontecer algum problema
- Quando complexo, a complexidade é justificada

Além disso, é importante também se atentar a como isso é resolvido em outras
organizações. Uma importante leitura é um artigo antigo, dos anos 2000 que se
chama [Bootstrapping an Infrastructure](http://www.infrastructures.org/papers/bootstrap/bootstrap.html).
Já naquela época se estabeleceu um sistema de gerência com os componentes que
hoje tem outro nome mas que são idênticos em natureza: um servidor "ouro" que
configura todos os serviços com controle de versão.

O que motivou essa proposta e busca por soluções foi a separação do servidor
de DHCP e DNS que são extremamente acoplados. Já estava claro que era necessário
estabelecer uma única fonte de verdade, e uma possibilidade seria mover os
arquivos de texto para o servidor de DHCP que atualizaria o DNS por DDNS. A
solução é coerente e funciona, mas continuar a gestão nos arquivos de texto
não é desejável devido aos problemas apontados anteriormente. Além disso,
deixo claro que de nenhuma forma a gestão discutida aqui exclui a atualização
do DNS pelo servidor DHCP, pois é uma estratégia que pode ser muito efetiva
principalmente para redes internas mais pra frente.

## NetBox

O [NetBox](https://netboxlabs.com/docs/netbox/en/stable/) é uma solução madura
de código aberto para modelar e documentar redes modernas. Ele combina gerenciamento
de IPs (IPAM) e gerenciamento de infraestrutura de data center (DCIM). É possível
estender todas as funcionalidades através de plugins e a sua principal funcionalidade
é interação fácil através de APIs com qualquer software. Fiquei sabendo da sua
existência conversando com o pessoal do POP-PR que está migrando a sua
infraestrutura para ele.

O NetBox é usado em várias e várias organizações, pois o modelo de rede que ele propõe
é muito bem feito e atende a muitos casos de uso diferentes. Nesse aspecto, encontrei uma
comunidade extremamente ativa e desenvolvedores pragmáticos sobre o que entra
e o que não entra no modelo padrão do NetBox. Assim, um aspecto extensionista
interessante para nós é que com certeza somos capazes de desenvolver ferramentas
que estendem o NetBox que serão muito úteis para toda a comunidade internacional.

A proposta do NetBox é que ele seja a Única Fonte de Verdade (SSoT), isto é,
qualquer pergunta que precisa ser feita sobre a rede, quem responde é o NetBox.
Importante notar porém que ele não é feito para ser uma representação do mundo
*atualizada em tempo real*. Ainda mais do que isso, o NetBox deve ser na verdade
a representação do mundo *ideal*, como ele deveria ser. Isto porque o fluxo deve
sempre ser: mudar o mundo da lua do NetBox, gerar as configurações baseadas
neste mundo e assim fazer com que o mundo real seja sincronizado com o mundo
que está no NetBox.

Então sobre os pontos chave que precisam ser atendidos temos:

- **Apresentação das informações é clara e 100% pesquisável:**

  Da melhor forma possível. Para navegar pela rede é recomendado usar sua interface
  web, mas interfaces em linha de comando como o [nbcli](https://github.com/nbcli/nbcli)
  ou interfaces customizadas utilizando a API em Python [pynetbox](https://github.com/netbox-community/pynetbox)
  devem ser feitas para atender as nossas necessidades. Como os administradores
  atuais tem familiaridade com arquivos texto em formatos específicos, é relativamente
  fácil de criar ferramentas de gestão que interoperam com o banco de dados do NetBox.

- **Consumível por máquinas para gerar arquivos de configuração:**

  Sim, sem problemas de sintaxe e semânticos pois os dados são fortemente tipados.

- **Integridade e consistência dos dados:**

  A fundação do NetBox é um modelo robusto do mundo real. Dispositivos podem ser
  parte de racks, estão localizados em salas e tem interfaces de energia e de
  rede que podem se ligar a outros dispositivos por meio de cabos.
  IPs são vinculados a essas interfaces e fazem parte de prefixos de rede que
  usa uma determinada VLAN. Esse emaranhado de relações permite contar a história
  completa da rede e impedir que exista informação duplicada e incoerente.

- **Concentrar todas as informações da rede, sem buracos**

  Será possível integrar todas as informações, e expandir com plugins caso seja
  interessante. Nativamente existe a provisão de contatos de responsáveis,
  máquinas físicas, máquinas virtuais, IPs, VLANs, rackeamento do data center,
  cabos de energia e conexões entre switches e patch panels. Com plugins temos
  [mapas de pontos de redes](https://github.com/netbox-community/netbox-floorplan-plugin),
  [gerenciamento de DNS](https://github.com/peteeckel/netbox-plugin-dns),
  [armazenamento de documentação](https://github.com/jasonyates/netbox-documents),
  [integração com Proxmox](https://github.com/netdevopsbr/netbox-proxbox),
  [visualizador de conexões](https://github.com/netbox-community/netbox-topology-views) e
  [mapa de discos](https://github.com/Zorlin/netbox-physical-storage). Não é necessário
  ir tão longe pra várias coisas porém pois a funcionalidade de criar campos customizados em objetos já é poderosa.

- **Controle de versão das alterações feitas e preferencialmente por quem, quando e talvez até por qual razão**

  O NetBox mantém um log de alterações contendo o responsável pela mudança e qual foi exatamente a mudança.
  E ao fazer a mudança, é possível especificar qual o motivo.

- **Etapas podem ser puladas em casos de emergência**

  Nada depende do NetBox funcionar. Ele é usado apenas como um banco de dados
  de informações, e se ele falhar, o que será impossível fazer será apenas gerar
  configurações novas de forma automática. Configurações manuais vão continuar
  sendo possíveis.

- **É fácil reverter quando acontecer algum problema**

  Em caso de alguma alteração massiva problemática, é possível reverter para uma
  versão anterior do banco de dados. Também faz parte do plano manter uma versão
  do NetBox auxiliar rodando que sempre tem informações do dia anterior para
  testar scripts ou reverter informações caso seja necessário. De toda forma note
  que alterar o NetBox não altera imediatamente o que é configurado por ele,
  pois existe uma camada de indireção.

- **Quando complexo, a complexidade é justificada**

  O modelo do NetBox realmente não é complexo e planejamos desativar todas as
  funcionalidades que no momento não são interessantes (como por exemplo a opção
  de ter vários grupos de VLANs diferentes). Pode ser trabalhoso a parte de
  inventoriar algumas das coisas (mas podemos ser vagos com os detalhes) porém
  é algo que os bolsistas com o tempo fazem e que a longo prazo só vai trazer benefícios.

## Automação: Ansible e Git

É evidente que o NetBox não faz nada por si só, ele é só um banco de dados.
Os mesmos "scripts" que liam os arquivos texto e geravam os arquivos de configuração
ainda vão ser pra sempre necessários e são a cola para permitir que o mundo
ideal do NetBox se materialize. Mas aí entram os sistemas de configuração que
já usamos fazem anos, como Salt e Ansible. O fluxo seria o seguinte:

```
.---------------.   .-----------------------.   .------------------.     .---------------------.
|   "Root cria" |   |   "NetBox cria commit"|   |    "Root avalia" | ok  |     "Root ou robô"  |
|  "informações"|-->|    "no Git com novas" |-->|   "configurações"|---->|   "implementa novas"|
|   "no NetBox" | ^ |     "configurações"   |   |     "criadas"    |     |    "configurações"  |
'---------------' | '-----------------------'   '------------------'     '---------------------'
                  |                                       | errado,
                  |                                       | arrumar!
                  '---------------------------------------'
```

Com essa camada de indireção usando Git também facilitamos reverter para um
estado anterior da configuração de algum serviço, basta reverter o commit e
implementar configurações antigas. E note que caso o servidor de Git ou o NetBox
esteja fora do ar, o administrador ainda pode alterar as configurações manualmente
e implementá-las usando Ansible (que precisa somente de acesso SSH).

Dependendo da periculosidade da operação, é possível que alguns tipos de mudanças
possam ser feitas automaticamente ou depois de um certo intervalo de tempo. Adições
de novos nomes não-conflitantes no DNS e no DHCP por exemplo pode ser considerada
uma operação de baixo risco. Alterações ou remoções nestes mesmos segmentos podem
exigir autorizações de mais um integrante da equipe de root para integrarem a
branch principal do Git por exemplo.

## Ajuste ao status quo

É compreensível que existam alguns óbvios pontos de discórdia.

### Eu não quero usar a interface web para mexer na rede

Pois bem, o NetBox tem uma API magnífica que você pode usar sem encostar
na interface web. O jeito mais simples de interagir com ela seria numa
sessão interativa do Python, eu já tenho esse script aqui:

```
#!/usr/bin/env -S python3 -i
import pynetbox
import os

nb = pynetbox.api(
    'https://netbox.c3sl.ufpr.br',
    token=os.environ['NETBOX_TOKEN']
)
```

Por exemplo se eu quiser saber informações sobre a máquina `estrella`:

```
>>> import yaml
>>> print(yaml.dump(dict(nb.dcim.devices.get(name='estrella'))))
```
```yaml
created: '2024-05-03T11:18:28.600413-03:00'
device_role:
  name: Gerencial
interface_count: 33
primary_ip4:
  address: 200.17.202.12/25
  url: https://netbox.c3sl.ufpr.br/api/ipam/ip-addresses/1334/
primary_ip6:
  address: 2801:82:80ff:8001:ba59:9fff:fe89:f251/64
  url: https://netbox.c3sl.ufpr.br/api/ipam/ip-addresses/1524/
rack:
  name: '7'
  url: https://netbox.c3sl.ufpr.br/api/dcim/racks/1/
site:
  name: "Departamento de Inform\xE1tica"
  url: https://netbox.c3sl.ufpr.br/api/dcim/sites/1/
status:
  value: active
url: https://netbox.c3sl.ufpr.br/api/dcim/devices/49/
```

Ou pode se usar algo como o [nbcli](https://github.com/nbcli/nbcli):

```console
$ pipx run nbcli search estrella

Device
======
Name      Status  Tenant  Site                         Rack  Role       Type  IP Address
estrella  Active  -       Departamento de Informática  7     Gerencial  1U    2801:82:80ff:8001:ba59:9fff:fe89:f251

Address
=======
IP Address                                Vrf  Status  Role  Tenant  DNS Name               Description
200.17.202.12/25                          -    Active  -     -       estrella.c3sl.ufpr.br  -
2801:82:80ff:8001:ba59:9fff:fe89:f251/64  -    SLAAC   -     -       estrella.c3sl.ufpr.br  -
```

De toda forma, creio que pode ser insuficiente para *algumas tarefas*.
Será necessário desenvolver um conjunto de ferramentas um pouco mais customizado
ao nosso gosto que pode se fundamentar no jeito que hoje as coisas são feitas.
O que proponho como design é uma ferramenta do tipo:

```console
$ netboxadm edit vlan xlab6
```

Que abre num editor um arquivo do gênero:
```yaml
xlab6:
  vid: 250
  prefix: 10.254.250.0/24
  gateway: 10.254.250.254
  ipv4s:
    2:
      host: estrella
    3:
      host: urquell
    11:
      host: l1
      mac: d0:94:66:bf:62:53
    12:
      host: l2
      mac: d0:94:66:bf:63:8f
    13:
      host: l3
      mac: d0:94:66:bf:5f:5b
    14:
      host: l4
      mac: d0:94:66:bf:62:37
    254:
      host: estrella
```

Que pode ser editado, sendo as alterações refletidas no NetBox apropriadamente
de forma análoga ao que temos hoje com os arquivos texto mas com os benefícios
de detectar nomes e MACs duplicados na rede inteira. Fazer essa interface é
curiosamente não muito difícil pois a parte de validar o que foi escrito fica
pro NetBox.

### E se o NetBox e/ou o sevidor Git falhar?

Você pode ajustar o arquivo de configuração que está na sua cópia local do
Git (ou copiar o arquivo da máquina) e alterar o que está na máquina manualmente
ou via Ansible. Note que isso não é diferente do que acontece se um script
que gera configuração a partir de arquivos texto falhar: você vai ter que
editar as configurações.

## Implementação do DHCP na estrella

Primeiro foram trazidos todos as VLANs da rede como mostra a figura abaixo. Cada
VLAN possui descrição e seus prefixos de rede associados.

![](/vlans.png)

Em seguida foram trazidos todos os prefixos de rede usados no DHCP da urquell.
Por exemplo o subconjunto de redes internas pode ser encontrado na figura abaixo.

![](/prefixes.png)

Foram criados campos customizados para o DHCP: domínios de busca, roteador, kernel
de boot, servidores DNS, servidor NTP, caminho do boot e servidor de boot além
do tempo de vida DHCP. Assim todos esses campos podem ser customizados por
prefixo de rede. Note que os campos são de objetos, então o roteador por exemplo
tem que ser um IP que existe dentro do NetBox para ser válido. E dentro de cada
prefixo são encontrados vários IPs mapeados como mostra a figura abaixo.

![](/ips.png)

Assim, é fácil ver qual IP está mapeado para qual máquina. Podemos configurar
campos específicos como o kernel de boot para algumas máquinas, e definir o
nome do host de cada uma delas. Essa informação é que será usada para definir
o DNS futuramente. E lembrando claro que cada IP é associado a uma interface
que tem um endereço MAC e a um dispositivo que pode ser colocado em um rack
do Data Center ou estar localizado em uma sala, contando com informações como
número de patrimônio, descrição, modelo e afins como mostra a figura abaixo.

![](/device.png)

Pois bem, para configurar o DHCP é uma tarefa simples: altera-se o campo
correspondente no NetBox e se gera um arquivo de configuração novo para o Kea
e também para configurar as interfaces de rede da estrella. Por exemplo uma parte
da configuração gerada para o Kea é a que segue:

```json
// VLAN 222 - xc3sl - Secretaria do C3SL
{
  "subnet": "10.254.222.0/24",
  "valid-lifetime": 3600,
  "option-data": [
    { "name": "domain-name-servers", "data": "200.17.202.3, 200.236.31.1" },
    { "name": "domain-search", "data": "c3local, c3sl.ufpr.br, inf.ufpr.br" },
    { "name": "extensions-path", "always-send": true, "data": "terminal" },
    { "name": "ntp-servers", "data": "200.17.202.3" },
    { "name": "routers", "data": "10.254.222.4" }
  ],
  "pools": [
    {
      "pool": "10.254.222.200-10.254.222.254",
      "user-context": { "nextbox_ip_range_id": 2 }
    }
  ],
  "reservations": [
    {
      "hostname": "swamp.c3local",
      "hw-address": "D0:94:66:BF:61:13",
      "ip-address": "10.254.222.11",
      "user-context": { "nextbox_ip_address_id": 950 }
    },
    {
      "hostname": "tempestade.c3local",
      "hw-address": "D0:94:66:BF:5D:A8",
      "ip-address": "10.254.222.13",
      "user-context": { "nextbox_ip_address_id": 951 }
    },
    {
      "hostname": "leffe.c3local",
      "hw-address": "00:BE:43:CE:1B:0F",
      "ip-address": "10.254.222.14",
      "user-context": { "nextbox_ip_address_id": 952 }
    },
    {
      "hostname": "rhalah.c3local",
      "hw-address": "00:BE:43:CE:1C:5C",
      "ip-address": "10.254.222.15",
      "user-context": { "nextbox_ip_address_id": 953 }
    },
    {
      "hostname": "bino.c3local",
      "hw-address": "64:1C:67:31:3F:CA",
      "ip-address": "10.254.222.16",
      "user-context": { "nextbox_ip_address_id": 954 }
    },
    {
      "hostname": "tuco.c3local",
      "hw-address": "F8:A9:63:65:6F:FA",
      "ip-address": "10.254.222.17",
      "user-context": { "nextbox_ip_address_id": 955 }
    },
    {
      "hostname": "talisker.c3local",
      "hw-address": "00:BE:43:CE:20:2F",
      "ip-address": "10.254.222.18",
      "user-context": { "nextbox_ip_address_id": 956 }
    }
  ],
  "id": 28
}
```

Podemos também por exemplo gerar arquivos texto automaticamente através dos
dados do NetBox, por exemplo no formato dos arquivos texto do mapa DNS/DHCP
da Urquell:

```
#
#  xc3sl - VLAN 222 (254 hosts)
#
#  Rede     : 10.254.222.0/24
#  Gateway  : 10.254.222.4
#  Broadcast: 10.254.222.255
#  IPs: de 10.254.222.1 a 10.254.222.254
#
#xc3sl| 2|[estrella]|NAOUSARNAOUSARNAO|terminal||
#xc3sl| 3|[urquell] |NAOUSARNAOUSARNAO|terminal||
#xc3sl| 4|[estrella]|NAOUSARNAOUSARNAO|terminal||
xc3sl |11|swamp     |d0:94:66:bf:61:13|terminal||PC Bona
xc3sl |13|tempestade|d0:94:66:bf:5d:a8|terminal||PC Ianka
xc3sl |14|leffe     |00:be:43:ce:1b:0f|terminal||PC Edemir
xc3sl |15|rhalah    |00:be:43:ce:1c:5c|terminal||PC Dell Castilho
xc3sl |16|bino      |64:1c:67:31:3f:ca|terminal||Micro de teste
xc3sl |17|tuco      |f8:a9:63:65:6f:fa|terminal||Laptop Castilho
xc3sl |18|talisker  |00:be:43:ce:20:2f|terminal||PC Dell Bona
```

## Oportunidades

O NetBox pode ser utilizados para fazer o mapeamento das máquinas em racks
e organizar e contabilizar os cabos que estão no Data Center. Não é uma tarefa
difícil, requer apenas algumas idas ao data center. Com a que a gente faz de
vez em quando já conseguimos posicionar as máquinas em racks como mostra a
figura abaixo.

![](/racks.png)

Outro aspecto é que é possível gerenciar os switches e suas VLANs. A partir
do NetBox é possível gerar também configurações para os switches de forma
automática, utilizando do modelo de dados que está presente. O esqueleto do
switch syrah é mostrado na figura abaixo.

![](/syrah.png)

Cada cabo e conexão feita pode ser rastreada. Por exemplo podemos ver que a
conexão entre a porta 6 do Syrah passa por dois patch panels até chegar na zara.

![](/trace.png)

Estaremos investigando plugins como o de mapas para mapear as salas graficamente
como hoje existem nos mapas da urquell em texto.

![](https://res.cloudinary.com/hl8zoliad/image/fetch/f_auto/https%3A%2F%2Fraw.githubusercontent.com%2Fnautobot%2Fnautobot-app-floor-plan%2Fdevelop%2Fdocs%2Fimages%2Ffloor-plan-populated.png)

Ou em grafos de topologia.

![](https://raw.githubusercontent.com/netbox-community/netbox-topology-views/develop/doc/img/topology_dark.png)

Enfim, resta experimentar mais. Estamos atualmente mapeando todas as máquinas
virtuais e contatos de responsáveis para melhorar o sistema de monitoramento
que hoje tem várias inconsistências que geram trabalho manual.
