---
title: duvel/Backup
---

Todo parque computacional está sujeito a ataques, erros humanos, falhas de
hardware, etc. Estes fatores podem levar a perda de dados, que às vezes são
inofensivos, mas às vezes podem custar alguns anos de pesquisa, perca de
dinheiro, contratos ou investimento.

Muitas vezes negligenciado, o backup é com certeza um dos serviços mais
importantes de toda organização. Além de bem implementado, é uma boa
prática fazer testes com o mínimo de frequência para garantir que este
esteja funcionando de maneira adequada.

## Arquitetura

Nosso backup principal está implementado na [](/pages/machines/base/duvel),
com segunda cópia física na ural, no data center da física.
Essa boa prática garante que, em caso de
danos físicos eventuais que o data center possa sofrer, a cópia dos dados ainda
esteja garantida em fisicamente em outro departamento.
Os backups e logs ficam em `/backup/`. Esse é um pool do ZFS do tipo "raidz2"
(igual ao RAID6).

## Implementação

Os seguintes arquivos fazem parte da gerência do Backup:

- `/usr/local/bin/backup` faz o backup propriamente.
- `/root/bin/cria-backup-{raiz,home}` cria arquivo de variáveis do
host/serviço.
- `/root/backups/` é onde as configurações de backup são guardadas.
- `/root/backups/raizes/DUMP-precmd` é o script de dump do Postgres que é feito automaticamente.
- `/root/backups/raizes/GLOBAL*` tem as configurações gerais do rsync e de exclusão.
- `/usr/local/bin/check_backup.sh` avisa os hosts que estão sem backup no
e-mail.
- `/home/salt/salt/excludes-vms.conf` VMs que serão ignoradas do script de
e-mail.

O script para fazer os backups roda automaticamente, dependendo da necessidade
de cada serviço, através do Crontab.

## Adicionar máquinas no Backup

O script de backup precisa ler um arquivo de variáveis da máquina/serviço.
Primeiro, criamos este arquivo da seguinte forma:

```bash
/root/bin/cria-backup-{raiz,home} <host>
```

Por exemplo, para fazer o backup da raiz da virtual 'asahi':

```bash
/root/bin/cria-backup-raiz asahi
```

Agora, é preciso determinar a necessidade do serviço e criar a entrada Cron
correspondente. Por exemplo, para colocar Backup diário do host asahi, editamos
o arquivo `/etc/crontab`.

```bash
# outras entradas acima
# ...
00 03   * * *   root    /usr/local/bin/backup /root/backups/raizes/asahi &
# ...
# outras entradas abaixo
```

Se não estiver familiarizado com a sintaxe, consulte a [página sobre o crontab](/pages/concepts/crontab).

Após adicionar a entrada, execute o comando dela uma vez para garantir que
que a duvel consegue se comunicar com o host e fazer o Backup propriamente:

```bash
/usr/local/bin/backup /root/backups/raizes/asahi &
[1] 15200
```

Utilizando o comando `jobs` você já consegue ter uma boa ideia se funcionou
propriamente ou não:

```bash
jobs
[1]-  Stopped                /usr/local/bin/backup /root/backups/raizes/fake &
[2]-  Running                /usr/local/bin/backup /root/backups/raizes/asahi &
```

Stopped provavelmente significa que há algum erro, [veja os erros comuns](#erros-comuns).

Observação: Se o serviço tiver a home separada da raiz, precisa de Backup da
raiz e da home. A maioria dos serviços tem backup apenas da raiz.

## Removendo hosts do backup

Além de remover a entrada da tabela cron, é recomendado em alguns casos copiar
o disco da máquina na Abbey. Se for o caso:

Na Abbey, use o vmcommand para pegar o ID do disco da máquina. Por exemplo,
para pegar o ID da codimd:

```bash
vmcommand show redmineprod
Name: redmineprod
ID: 119
...
```

Use o `rbd` (rados block device) para listar os discos e mapear o disco na
abbey. Neste caso, estamos atrás do disco de ID 19:

```bash
rbd list
...
vm-118-disk-0
vm-119-disk-0
vm-121-disk-0
vm-122-disk-0
...
```

```bash
rbd map vm-119-disk-0
/dev/rbd0
```

Primeiro, cheque se já algo montado em `/mnt`. Você ainda, pode conferir
informações sobre o disco mapeado com `lsblk` ou `fdisk -l`.

```bash
mount | grep "/mnt"
```

Se estiver tudo de acordo, monte o disco em `/mnt`.

```bash
mount /dev/rdb0 /mnt
```

Agora, em `/root/virtuais/servicos/cemiterio` crie um diretório com o nome do
host e faça um rsync do `/mnt` para o novo diretório.

```bash
rsync -ahHPv /mnt/ /root/virtuais/servicos/cemiterio/redmineprod
...
var/tmp/.root_8a1b4e_salt/running_data/var/cache/salt/minion/proc/
var/tmp/.root_8a1b4e_salt/running_data/var/cache/salt/minion/roots/
var/tmp/.root_8a1b4e_salt/running_data/var/cache/salt/minion/roots/mtime_map
            326 100%    2.82kB/s    0:00:00 (xfr#85716, to-chk=10/108561)
```

Desmonte o disco, e use o rbd para desmapear.

```
umount /mnt
rbd unmap /dev/rbd0
```

Na interfaceweb, destrua o host.

## Ignorando máquinas do Backup

Caso uma máquina não tenha necessidade de backup, por ser uma máquina teste
por exemplo, devemos adicionar na lista de vms ignoradas.

Logando na duvel como usuário salt, você pode alterar o arquivo
`/home/salt/salt/excludes-vms.conf` e acrescentar o nome da vm a ser ignorada.

## Recuperando backups

Através do comando:

```console
$ rsync --numeric-ids --info=progress2 -avHh <backup_path> <destino>
```

Pode ser usado com nohup para não ficar atrelado ao terminal.

## Erros comuns

Há duas formas mais básicas de debugar:

### Logs

Olhar o log do e-mail ou o log do Backup da vm que é criado em
`/backup/<host>/raiz/logs/`. Alguns dos erros mais comuns:

- lock existente: delete o arquivo de lock, criado em /backup/\<host\>/raiz/LOCK.
- erro 255: provavelmente a duvel não consegue criar uma shell remota
(protocolo SSH) com a máquina.
- erro 23: transferência parcial. Algum erro de comunicação durante
o rsync.

### Rsync

Tentar abrir uma shell remota da duvel até o host(protocolo SSH). Se
conseguir, tentar rsync na mão e ver o que acontece.
